# pseudo_image.tcl

# proc create_main_image { win image xscroll yscroll } 
# proc create_image_canvas { var win name ringlist } 
# proc create_image_rings { var win name image ringlist} 
# proc highlight_image_ring { imagewin ringnum ringlist }  
# proc unhighlight_image_ring { imagewin ringnum ringlist } 
# proc draw_image { var1 var2 win name image xscroll yscroll ringlist} 
# proc bindMouseToImage { var1 var2 win name image xscroll yscroll ringlist } 
# proc execute_bindMouseToImage { var1 var2 x y win ringlist } 

# -------------------------------------------------------------------

# Creates a frame to hold the pseudo image.
proc create_main_image { win image xscroll yscroll } {
   frame $win -borderwidth 2 -relief ridge 
   scrollbar $win.$xscroll \
   	-orient horizontal \
   	-command [subst {[list BindXview \
                "$win.$image" ] \
                }] 
   scrollbar $win.$yscroll \
   	-orient vertical \
   	-command [subst {[list BindYview \
                "$win.$image" ] \
                }] 
}

# Creates the canvas to hold the pseudo image.
proc create_image_canvas { var win name ringlist } {
    upvar #0 $var SkelPlot

    set scale $SkelPlot(scale)
    set boxes $SkelPlot(unitboxes,skelplot)
    upvar #0 $SkelPlot(sample,$name,serieslist) serieslist
    set count [llength $serieslist]
    
    upvar #0 $ringlist thelist
    set width 0
    foreach item $thelist {
       upvar #0 $item theelement
       set width [expr $theelement(width) + $width]
    }
    set width [expr $width * 50 + 50 ]
    set height 200
    canvas $win -width $width -height $height \
	    -highlightthickness 0 \
	    -background gray25 \
	    -confine true \
	    -cursor target \
	    -xscrollincrement $scale \
	    -scrollregion "0 0 $width $height" 
       
}

# Places pseudo rings on the main image canvas at intervals 
# of 50 * ring boundary width.
proc create_image_rings { var win name image ringlist} {
   upvar #0 $var SkelPlot

   set scale $SkelPlot(scale)
   set boxes $SkelPlot(unitboxes,skelplot)
   set invert $SkelPlot(sample,$name,invert)

   upvar #0 $SkelPlot(sample,$name,serieslist) serieslist

   set count [llength $serieslist]

   set startindex [lindex $serieslist 0]
   upvar #0 $startindex array
   set startyear $array(year)
   set offset [expr $startyear % 5]

   set height [$win cget -height]
   set width [$win cget -width]

   # draw vertical lines 
   upvar #0 $ringlist thelist
   set y0 0
   set y1 $height
   set sclX 0
#   set textX 0
   for {set x 0} { $x < $count } { incr x } {
      set item [lindex $thelist $x]
      upvar #0 $item theelement
#      set textX [expr $theelement(width) * 25 + $sclX]
	   set sclX [expr int($theelement(width)/6 + $sclX)+1]
#       puts "$x $sclX"
      if {[lsearch -exact $theelement(typelist) removed]>=0} {
   	   $win create line $sclX $y0 $sclX $y1 \
	   	    -fill blue -tags [list "Boundary-$x" "removed"]            
      } elseif {[lsearch -exact $theelement(typelist) added]>=0} {
   	   $win create line $sclX $y0 $sclX $y1 \
	   	    -fill yellow -tags [list "Boundary-$x" "added"]            
      } else {
   	   $win create line $sclX $y0 $sclX $y1 \
	   	    -fill green -tags [list "Boundary-$x" "normal"]      
      }    
	   if {[expr ($x+$offset) % 5] == 0} {
		   set textid [$win create text $sclX [expr $height - 10] \
                 -text [expr $x+$startyear] -tag text -fill red]

	      # Put a gray background behind text so easier to read
	      foreach {l t r b} [$win bbox $textid] {}
	      $win create rectangle $l $t $r $b -fill gray \
		       -width 1 -outline gray -tag textbg
         $win raise textbg
         $win raise text
	   } 
    } 
}

# Highlights the rings in the image display based on the ring
# selected in the skeleton plot.
proc highlight_image_ring { imagewin ringnum ringlist }  {
   global SkelPlot boundarycolor   

   set RBoundary [getRBoundary $ringlist $ringnum]
   set LBoundary [getLBoundary $ringlist $ringnum]
   set SkelPlot(tags,highlight) [list $RBoundary $LBoundary]
   $imagewin itemconfigure $RBoundary -fill $boundarycolor(highlight) 
   $imagewin itemconfigure $LBoundary -fill $boundarycolor(highlight) 
   .trees_main.zoompanel.zoomframe.canvas itemconfigure $RBoundary -fill $boundarycolor(highlight)
   .trees_main.zoompanel.zoomframe.canvas itemconfigure $LBoundary -fill $boundarycolor(highlight)   

}

# Returns the rings in the image display to their original
# colors after highlighting is no longer desired.
proc unhighlight_image_ring { imagewin ringnum } {
   global boundarycolor
   
   $imagewin itemconfigure removed -fill $boundarycolor(removed)
   $imagewin itemconfigure added -fill $boundarycolor(added)
   $imagewin itemconfigure normal -fill $boundarycolor(normal)
   .trees_main.zoompanel.zoomframe.canvas itemconfigure removed -fill $boundarycolor(removed)
   .trees_main.zoompanel.zoomframe.canvas itemconfigure added -fill $boundarycolor(added)
   .trees_main.zoompanel.zoomframe.canvas itemconfigure normal -fill $boundarycolor(normal)    

}

# Draws the pseudo image to the main screen.
proc draw_image { var1 var2 win name image xscroll yscroll ringlist} {

   create_main_image $win $image $xscroll $yscroll
   create_image_canvas $var1 $win.$image $name $ringlist
   configure_xscroll $win $image $xscroll
   configure_yscroll $win $image $yscroll
   create_image_rings $var1 $win.$image $name $image $ringlist
#   bindMouseToImage $var1 $var2 $win $name $image $xscroll $yscroll $ringlist

   grid $win.$image -row 0 -column 0 -sticky new
   grid $win.$xscroll -row 1 -column 0 -sticky new
   grid $win.$yscroll -row 0 -column 1 -sticky nes
   grid $win -row 0 -column 0 -sticky new
   grid columnconfigure $win 0 -weight 1
   grid rowconfigure . 0 -weight 1

}

