# Procedure trees_skelplot_create creates the initial skeleton
# plot displayed on the screen.
proc trees_skelplot_create {} {

   # Global variables and settings
   global SkelPlot
   global Image
   global session mosaic findrings measurements skelplot boundarycolor
   global needSave noprompting labels gui

   if {$measurements(status) != "COMPLETE"} {
       messagebox "SkelPlot Error" "Cannot create skeleton plot because ring measurements are not complete."
   } elseif {$skelplot(status) == "COMPLETE"} {
       messagebox "SkelPlot Info" "Skeleton plot has already been generated."
   } else {
      set skelplot(status) ACTIVE
      trees_update_status

      # puts "ringwidths: $measurements(ringwidth)"

      set seriesname "series"
      set SkelPlot(sample,$seriesname,color) $boundarycolor(timeseries)
      set SkelPlot(sample,$seriesname,invert) 0
      if {![info exists skelplot(endyear)]} {
         set skelplot(endyear) 0
      }      
      if {![info exists skelplot(seriesname)]} {
         set name [split $session(directory) "/"]
         set skelplot(seriesname) [lindex $name end]
      }

      # Initialize skelplot variables
      skelplot_init SkelPlot

      # Initialize Image variables
      image_init Image

      # Load the measured ringwidths
      load_ringwidth_series  SkelPlot $measurements(ringwidth) $seriesname 

      # print_series_info $SkelPlot(sample,$name,serieslist)

      # Create mouse button-1 click binding for the browse image
      # to find coordinates and tags at click location
      bindMouseToImage SkelPlot Image .trees_main.imagepanel $seriesname canvas xsbar ysbar $SkelPlot(sample,$seriesname,masterlist)

      # Create initial skeleton plot canvas
      draw_skelplot SkelPlot .trees_main.skelplotpanel topmarker timeseries $seriesname bottommarker xscr .trees_main.imagepanel.canvas

      # Finish up
	  set gui(skelplot) 1
	  set skelplot(status) COMPLETE
	  trees_update_status
    
   }
}



proc trees_create_skelplotpanel {win} {
    frame ${win} -borderwidth 2 -relief ridge 
}

proc trees_destroy_skelplot { win } {

   destroy $win.topmarker
   destroy $win.timeseries
   destroy $win.series
   destroy $win.bottommarker
   destroy $win.xscr
   bind .trees_main <Any-Key> {}

}


# Creates a <button-1> mouse binding in the image canvas.
proc bindMouseToImage { var1 var2 win name image xscroll yscroll ringlist } {

   bind "$win.$image" <Button-1> \
      "execute_bindMouseToImage $var1 $var2 %x %y $win.$image $ringlist"
}

# Updates the global $Image(location) variable each time <button-1>
# is clicked within the image canvas.
proc execute_bindMouseToImage { var1 var2 x y win ringlist } {

   upvar #0 $var1 SkelPlot
   upvar #0 $var2 Image

   # Figure out where the current view is to map coordinates
   set xwindowoffset [expr [lindex [$win xview] 0]*[$win cget -width]]
   set ywindowoffset [expr [lindex [$win yview] 0]*[$win cget -height]]

   # Set the search area to 1 pixel right and left, 0 pixel up and down
   set x0 [expr $x - 1 + $xwindowoffset]
   set x1 [expr $x + 1 + $xwindowoffset]
   set y0 [expr $y + $ywindowoffset]
   set y1 [expr $y + $ywindowoffset]   

   # Find tags at this location
   set ROIitems [$win find overlapping $x0 $y0 $x1 $y1]
#   puts $ROIitems
   # Build the Image(location) variable [ x y tag1 tag2 ... ]
   set Image(location) [list $x $y]
   if {[llength $ROIitems] > 0 } {
      foreach item $ROIitems {
         set tags [$win gettags $item]
         foreach tag $tags {
            lappend Image(location) $tag
         }
      }
   }
   puts "Imagelocation: $Image(location)"
}
