# trees_skelplot_dialog.tcl - Skeleton Plot Modification Dialog Boxes:

# Procedure Index:

# proc mstatusbar { object statusmesg } 
# proc bstatusbar { win object statusmesg } 
# proc cstatusbar { object statusmesg } 
# proc select_ring_number { dlg skelplotwin series top bottom timeseries imagecanvas} 
# proc move_ring_select { dlg imagecanvas skelplotwin series top bottom timeseries } 
# proc redraw_zoom_rings { zoomwin }
# proc display_attributes_dialog { dlg } 
# proc changeRingNotes {} 
# proc resetRingNotes {} 
# proc change_cutoff { var dlg win top timeseries seriesname bottom scroll imagecanvas } 
# proc trees_set_endyear { var1 var2 dlg imagewin plotwin top timeseries series bottom scroll }
# proc trees_set_startyear { var1 var2 dlg imagewin plotwin top timeseries series bottom scroll }
# proc calc_end_year { var1 var2 dlg imagewin plotwin top timeseries series bottom scroll }
# --------------------------------------------------------------------------

# display message in main window only
proc mstatusbar { object statusmesg } {
   bind $object <Enter> [subst {
      global mainstatus   
      set mainstatus "$statusmesg"
   }] 
   bind $object <Leave> [subst {
      global mainstatus
      set mainstatus ""
   }] 
   bind $object <Destroy> [subst {
      global mainstatus
      set mainstatus ""
   }] 
   bind $object <Unmap> [subst {
      global mainstatus
      set mainstatus ""
   }] 
}

# display message in both main window and appropriate option box
proc bstatusbar { win object statusmesg } {
   bind $object <Enter> [subst {
      global mainstatus plot 
      set mainstatus "$statusmesg"   
      set plot($win-status) "$statusmesg"      
   }] 
   bind $object <Leave> [subst {
      global mainstatus plot
      set mainstatus ""
      set plot($win-status) ""
   }] 
   bind $object <Destroy> [subst {
      global mainstatus plot
      set mainstatus ""
      set plot($win-status) ""
   }] 
   bind $object <Unmap> [subst {
      global mainstatus plot
      set mainstatus ""
      set plot($win-status) ""
   }] 
}

# display message in both cross-dating window and main window
proc cstatusbar { object statusmesg } {
     bind $object <Enter> [subst {
      global mainstatus crdatestatus 
      set mainstatus "$statusmesg"   
      set crdatestatus "$statusmesg"
   }] 
   bind $object <Leave> [subst {
      global mainstatus plot
      set mainstatus ""
      set crdatestatus ""
   }] 
   bind $object <Destroy> [subst {
      global mainstatus plot
      set mainstatus ""
      set crdatestatus ""
   }] 
   bind $object <Unmap> [subst {
      global mainstatus plot
      set mainstatus ""
      set crdatestatus ""
   }] 
}


# Procedure select_ring_number provides the user with a dialog boxfor
#  selecting a ring by year number.

proc select_ring_number { dlg skelplotwin series top bottom timeseries imagecanvas} {
   
   global SkelPlot

   # Create basic Dialog Box

   dialog_create "Select Ring" $dlg 

   label $dlg.info.ringsLbl -text "Enter the Ring's year number or,\n Click in the Skeleton Plot" \
      -bg LightGray -fg black
   entry $dlg.info.rings -width 5 -textvariable SkelPlot(request) -bg White -fg #0000C8 
   mstatusbar $dlg.info.rings "Ring Year Number Chosen"
   
   grid $dlg.info.ringsLbl $dlg.info.rings -sticky new -padx 4
   grid columnconfigure $dlg 0 -minsize 275

   # Select and Cancel buttons
   button $dlg.controls.select -text "Select" -bg LightGray -fg black\
   -command [subst {
      move_ring_select $dlg $imagecanvas $skelplotwin $series $top $bottom $timeseries
      }]
   
#     Moved the following code to move_ring_select procedure:   
#	  -command [subst {
#        global SkelPlot
#        global ringSelect
#        set SkelPlot(request) [getIndex $SkelPlot(sample,$series,serieslist) $SkelPlot(request)]
#        if {\$SkelPlot(request) >= 0 && \
#            \$SkelPlot(request) < \$SkelPlot(size,display)} {           
#           unhighlight_image_ring $imagecanvas \$SkelPlot(count,current)
#           set SkelPlot(count,current) \$SkelPlot(request) 
#           select_ring $skelplotwin $series $top $bottom $timeseries $imagecanvas
#           set ringSelect \$SkelPlot(count,current) 
#           destroy $dlg                  
#        }
#       }]

   mstatusbar $dlg.controls.select \
      "Click Select to select ring year number."
   
   button $dlg.controls.cancel -text "Cancel" -bg LightGray -fg black\
     -command [subst { 
        global ringSelect
        set ringSelect -1
        destroy $dlg
     }] 
   mstatusbar $dlg.controls.cancel \
      "Click Cancel to cancel selecting a ring" 
      
   wm protocol $dlg WM_DELETE_WINDOW "$dlg.controls.cancel invoke"  
   
   grid $dlg.controls.select -row 1 -column 0 
   grid $dlg.controls.cancel -row 1 -column 1
#   pack $dlg.controls.select $dlg.controls.cancel -expand yes -side left
}

# move_ring_select is invoked when the user wants to select a ring by year number
# The procedure will cause the currently highlighted ring to move to the
# year requested by the user if a valid year is entered, otherwise no change.
proc move_ring_select { dlg imagecanvas skelplotwin series top bottom timeseries } {
   global SkelPlot
   global ringSelect
   set index [getIndex $SkelPlot(sample,$series,serieslist) $SkelPlot(request)]
   if { $index >= 0 && $index < $SkelPlot(size,display) } {           
      unhighlight_image_ring $imagecanvas $SkelPlot(count,current)
      set SkelPlot(count,current) $index 
      select_ring $skelplotwin $series $top $bottom $timeseries $imagecanvas
      set ringSelect $SkelPlot(count,current) 
      destroy $dlg                  
   } else {
      set SkelPlot(request) [getYear $SkelPlot(sample,$series,serieslist) $SkelPlot(count,current)]
   }
}


# This procedure removes, then redraws tree ring boundaries in the 
# zoom window.
proc redraw_zoom_rings { zoomwin } {
   global findrings zoom boundarycolor

   $zoomwin delete "ring"
   Sadie_TREES_DisplayPartRings $findrings(loaded,chainmap) \
      $zoom(X1) $zoom(Y1) $zoom(X2) $zoom(Y2) $zoomwin
}

# Procedure display_attributes_dialog displays the attributes of the current ring
# (as determined by the global variable $SkelPlot(count,current).
proc display_attributes_dialog { dlg } {

   global SkelPlot skelplot

   dialog_create "Ring Attributes" $dlg

   label $dlg.info.sampleLbl -text "Sample Name:" -bg LightGray -fg black
   entry $dlg.info.sample -width 30 -state disabled -textvariable skelplot(seriesname) -bg white -fg #0000C8
   mstatusbar $dlg.info.sample "The Sample Name"   
   grid $dlg.info.sampleLbl -column 0 -row 0 -sticky nes -padx 4
   grid $dlg.info.sample -column 1 -row 0 -sticky nws -padx 4

   label $dlg.info.ringLbl -text "Ring Number:" -bg LightGray -fg black
   entry $dlg.info.ring -width 12 -state disabled -textvariable SkelPlot(count,current) -bg white -fg #0000C8
   mstatusbar $dlg.info.ring "The Ring Number"   
   grid $dlg.info.ringLbl -column 0 -row 1 -sticky nes -padx 4
   grid $dlg.info.ring -column 1 -row 1 -sticky nws -padx 4
   
   label $dlg.info.yearLbl -text "Year Number:" -bg LightGray -fg black
   entry $dlg.info.year -width 12 -state disabled -textvariable SkelPlot(count,year) -bg white -fg #0000C8
   mstatusbar $dlg.info.year "The Year Number"   
   grid $dlg.info.yearLbl -column 0 -row 2 -sticky nes -padx 4
   grid $dlg.info.year -column 1 -row 2 -sticky nws -padx 4

   label $dlg.info.lbLbl -text "Left Boundary:" -bg LightGray -fg black
   entry $dlg.info.lb -width 12 -state disabled -textvariable SkelPlot(count,leftboundary) -bg white -fg #0000C8
   mstatusbar $dlg.info.lb "The Ring's Left Boundary"   
   grid $dlg.info.lbLbl -column 0 -row 3 -sticky nes -padx 4
   grid $dlg.info.lb -column 1 -row 3 -sticky nws -padx 4
   
   label $dlg.info.rbLbl -text "Right Boundary:" -bg LightGray -fg black
   entry $dlg.info.rb -width 12 -state disabled -textvariable  SkelPlot(count,rightboundary) -bg white -fg #0000C8
   mstatusbar $dlg.info.rb "The Ring's Right Boundary"   
   grid $dlg.info.rbLbl -column 0 -row 4 -sticky nes -padx 4
   grid $dlg.info.rb -column 1 -row 4 -sticky nws -padx 4
   
   label $dlg.info.widthLbl -text "Ring Width:" -bg LightGray -fg black
   entry $dlg.info.width -width 12 -state disabled  -textvariable SkelPlot(count,width) -bg white -fg #0000C8
   mstatusbar $dlg.info.width "The measured width of the ring in pixels"   
   grid $dlg.info.widthLbl -column 0 -row 5 -sticky nes -padx 4
   grid $dlg.info.width -column 1 -row 5 -sticky nws -padx 4
   
   label $dlg.info.indexvalLbl -text "Index Value:" -bg LightGray -fg black
   entry $dlg.info.indexval -width 12 -state disabled  -textvariable SkelPlot(count,indexval) -bg white -fg #0000C8
   mstatusbar $dlg.info.indexval "The de-trended index value of the ring's width"   
   grid $dlg.info.indexvalLbl -column 0 -row 6 -sticky nes -padx 4
   grid $dlg.info.indexval -column 1 -row 6 -sticky nws -padx 4
   
   label $dlg.info.bonelengthLbl -text "Bone Length:" -bg LightGray -fg black
   entry $dlg.info.bonelength -width 12 -state disabled -textvariable SkelPlot(count,bonelength) -bg white -fg #0000C8
   mstatusbar $dlg.info.bonelength "The length of the skeleton plot bone"   
   grid $dlg.info.bonelengthLbl -column 0 -row 7 -sticky nes -padx 4 
   grid $dlg.info.bonelength -column 1 -row 7 -sticky nws -padx 4            

   label $dlg.info.typeLbl -text "Ring Types:" -bg LightGray -fg black
   entry $dlg.info.type -width 30 -state disabled -textvariable SkelPlot(count,typelist) -bg white -fg #0000C8
   mstatusbar $dlg.info.type "The ring's types"   
   grid $dlg.info.typeLbl -column 0 -row 8 -sticky nes -padx 4 
   grid $dlg.info.type -column 1 -row 8 -sticky nws -padx 4       

   frame $dlg.info.sep -height 2 -borderwidth 1 -relief sunken -background LightGray
   grid $dlg.info.sep -column 0 -columnspan 2 -row 9 -sticky new -pady 2

   label $dlg.info.textLbl -text "Analyst Notes:" -bg LightGray -fg black
   button $dlg.info.save -text "Save\nNotes" -bg LightGray -fg black \
      -command {  
          changeRingNotes
#         print_ring_info
       }
   button $dlg.info.reset -text "Reset\nNotes" -bg LightGray -fg black \
      -command {
          resetRingNotes
       }
   text $dlg.info.notes -height 7 -width 30 -wrap word -bg white -fg #0000C8 \
      -yscrollcommand "$dlg.info.ysbar set" 
   scrollbar $dlg.info.ysbar -orient vertical \
       -background LightGray  \
       -command "$dlg.info.notes yview" 
   mstatusbar $dlg.info.save "Click Save Notes to save analyst notes for this ring."
   mstatusbar $dlg.info.reset "Click Reset Notes to revert to previous analyst notes for this ring."
   mstatusbar $dlg.info.notes "Type Analyst Notes in text area for this ring."   
   grid $dlg.info.textLbl -column 0 -row 10 -sticky nes -padx 4
   grid $dlg.info.save -column 0 -row 11 -sticky new -padx 4
   grid $dlg.info.reset -column 0 -row 11 -sticky sew -padx 4   
   grid $dlg.info.notes $dlg.info.ysbar -column 1 -row 11 -sticky nes -padx 4
   
   $dlg.info.notes insert 1.0 $SkelPlot(count,notes)

#   grid columnconfigure $dlg.info 0 -minsize 200
#   grid columnconfigure $dlg.info 1 -minsize 200

   # Dismiss button
   button $dlg.controls.dismiss -text "Close Ring Attributes Window" -bg LightGray -fg black\
     -command [subst { 
        destroy $dlg
     }] 
   mstatusbar $dlg.controls.dismiss \
      "Closes the Ring Attributes window." 
      
   wm protocol $dlg WM_DELETE_WINDOW "$dlg.controls.dismiss invoke"  
   
   grid $dlg.controls.dismiss -row 1 -column 0
}

# Changes the Analyst Notes associated with a ring when the "SAVE" button
# is pressed from the Ring Attributes Dialog Box
proc changeRingNotes {} {
   global SkelPlot
   
   set notes [string trimright [.trees_main.attributes.info.notes get 1.0 end]]
   set notes [split $notes \n]
#   set notes [list $notes]
#   puts "ChangeRingNotes: $notes"
   setRingNotes $SkelPlot(count,current) $notes
   set $SkelPlot(count,notes) $notes
   select_ring .trees_main.skelplotpanel series topmarker bottommarker timeseries .trees_main.imagepanel.canvas 
   redraw_skelplot SkelPlot .trees_main.skelplotpanel topmarker timeseries series bottommarker xscr .trees_main.imagepanel.canvas
      
}

# Resets the Analyst Notes associated with a ring when the "RESET" button
# is pressed from the Ring Attributes Dialog Box to its previous value.
proc resetRingNotes {} {
   global SkelPlot
   
   .trees_main.attributes.info.notes delete 1.0 end
   .trees_main.attributes.info.notes insert 1.0 $SkelPlot(count,notes)   
}





# Test Code for printing Ring Information
proc print_ring_info {} {
   global SkelPlot skelplot

   set text [get_sorted_array SkelPlot]
   puts "SkelPlot Array:\n$text"

   print_series_info $SkelPlot(sample,series,serieslist)

}

# Procedure change_cutoff allows the user to manipulate the absolute and
# first difference cutoff values used to calculate the skeleton plot.
proc change_cutoff { var dlg win top timeseries seriesname bottom scroll imagecanvas } {

   upvar #0 $var SkelPlot
   upvar #0 $SkelPlot(sample,$seriesname,serieslist) theWorkingList
   
   dialog_create "Change Skeleton Plot Cutoffs" $dlg 
   

   label $dlg.info.sizeLbl -text "Number of Rings:" -bg LightGray -fg black
   entry $dlg.info.size -textvariable SkelPlot(count,rings) -state disabled -bg white -fg #0000C8
   mstatusbar $dlg.info.size "Number of Rings on Skeleton Plot"
   
   grid $dlg.info.sizeLbl $dlg.info.size -sticky new -padx 4
   grid columnconfigure $dlg 0 -minsize 275
   
   label $dlg.info.abLbl -text "Absolute Cutoff:" -bg LightGray -fg black
   label $dlg.info.diffLbl -text "First Difference:" -bg LightGray -fg black
   
   scale $dlg.info.abCutoff -from 2 -to 10 -bg LightGray -fg black\
     -orient horizontal \
     -variable SkelPlot(abCutoff)
   scale $dlg.info.diffCutoff -from 2 -to 10 -bg LightGray -fg black\
     -orient horizontal \
     -variable SkelPlot(diffCutoff)   
   
   grid $dlg.info.abLbl $dlg.info.abCutoff -sticky new -padx 4
   grid $dlg.info.diffLbl $dlg.info.diffCutoff -sticky new -padx 4      

   # Select Default Cancel buttons
   button $dlg.controls.redraw -text "Redraw Skeleton Plot" -bg LightGray -fg black\
	  -command [subst {
               compute_bones SkelPlot $SkelPlot(sample,$seriesname,serieslist)
               redraw_skelplot SkelPlot $win $top $timeseries $seriesname $bottom $scroll $imagecanvas
              }]
   mstatusbar $dlg.controls.redraw \
      "Click to redraw Skeleton Plot"

   button $dlg.controls.default -text "Default Values" -bg LightGray -fg black\
	  -command [subst {
               set SkelPlot(abCutoff) 2
               set SkelPlot(diffCutoff) 2
               compute_bones SkelPlot $SkelPlot(sample,$seriesname,serieslist)
               redraw_skelplot SkelPlot $win $top $timeseries $seriesname $bottom $scroll $imagecanvas
              }]
   mstatusbar $dlg.controls.default \
      "Click to reset cutoffs to default values"
         
   button $dlg.controls.cancel -text "Dismiss" -bg LightGray -fg black\
     -command [subst { 
        destroy $dlg
     }] 
   mstatusbar $dlg.controls.cancel \
      "Click Dismiss to close this window" 
      
   wm protocol $dlg WM_DELETE_WINDOW "$dlg.controls.cancel invoke"  
   
   grid $dlg.controls.redraw -row 1 -column 0 
   grid $dlg.controls.default -row 1 -column 1
   grid $dlg.controls.cancel -row 1 -column 2
}

# Procedure creates a user dialog for setting the ending year for the
# skeleton plot and sets the global variable skelplot(endyear)
proc trees_set_endyear { var1 var2 dlg imagewin plotwin top timeseries series bottom scroll } {
   global SkelPlot skelplot 

   # Create basic Dialog Box

   dialog_create "Set Skeleton Plot End Year" $dlg 
   puts "dlg is: $dlg"
   label $dlg.info.endyearLbl -text "Enter the year corresponding to the last ring" \
      -bg LightGray -fg black
   entry $dlg.info.endyear -width 8 -textvariable SkelPlot(endyear) -bg White -fg #0000C8 
   mstatusbar $dlg.info.endyear "Year corresponding to last ring in skeleton plot"
   label $dlg.info.resetLbl \
      -text "(Enter 0 to reset ring year to ring number)" \
      -bg LightGray -fg black
   
   grid $dlg.info.endyearLbl $dlg.info.endyear -sticky new -padx 4
   grid $dlg.info.resetLbl -sticky new -padx 4
   grid columnconfigure $dlg 0 -minsize 275

   # Setendyear and Cancel buttons
   button $dlg.controls.setendyear -text "Set End Year" -bg LightGray -fg black\
	  -command [ subst {
                reset_end_year $var1 $var2 $dlg $imagewin $plotwin $top $timeseries $series $bottom $scroll 
                }]

   mstatusbar $dlg.controls.setendyear \
      "Click Set to set skeleton plot end year."
   
   button $dlg.controls.cancel -text "Cancel" -bg LightGray -fg black\
     -command [ subst {  
               destroy $dlg
              }]
              
   mstatusbar $dlg.controls.cancel \
      "Click Cancel to cancel setting the skeleton plot end year." 
      
   wm protocol $dlg WM_DELETE_WINDOW "$dlg.controls.cancel invoke"  
   
   grid $dlg.controls.setendyear -row 1 -column 0 
   grid $dlg.controls.cancel -row 1 -column 1
   
}

# Procedure reset_end_year performs the actual reset of the skeleton plot after
# the user has selected a new endyear.
proc reset_end_year { var1 var2 dlg imagewin plotwin top timeseries series bottom scroll } {

   global SkelPlot skelplot global_endyear

   # Make sure that the endyear is a proper integer number
   set result [regexp {(^\-[0-9]+$)|(^[0-9]+$)} $SkelPlot(endyear) ]   
   if {$result} {
      # Reset the global variable skelplot(endyear)
      set skelplot(endyear) $SkelPlot(endyear)
      set global_endyear $SkelPlot(endyear)
      # Recompute indices, bones for the skeleton plot.   
      compute_indices $var1 $SkelPlot(sample,$series,masterlist) $SkelPlot(sample,$series,serieslist) $series
      compute_bones $var1 $SkelPlot(sample,$series,serieslist)

      # Re-draw the skeleton plot.
      redraw_skelplot $var1 $plotwin $top $timeseries $series $bottom $scroll $imagewin   

      # Destroy this dialog box           
      destroy $dlg
   } else {
       set SkelPlot(endyear) $skelplot(endyear)
   }      
}

# Procedure creates a user dialog for setting the starting year for the
# skeleton plot
proc trees_set_startyear { var1 var2 dlg imagewin plotwin top timeseries series bottom scroll } {
   global SkelPlot skelplot

   # Create basic Dialog Box

   dialog_create "Set Skeleton Plot Start Year" $dlg 

   label $dlg.info.startyearLbl -text "Enter the year corresponding to the first ring" \
      -bg LightGray -fg black
   entry $dlg.info.startyear -width 8 -textvariable SkelPlot(startyear) -bg White -fg #0000C8 
   mstatusbar $dlg.info.startyear "Year corresponding to first ring in skeleton plot"
   label $dlg.info.resetLbl \
      -text "(Enter 0 to reset ring year to ring number)" \
      -bg LightGray -fg black
   
   grid $dlg.info.startyearLbl $dlg.info.startyear -sticky new -padx 4
   grid $dlg.info.resetLbl -sticky new -padx 4
   grid columnconfigure $dlg 0 -minsize 275

   # Setstartyear and Cancel buttons
   button $dlg.controls.setstartyear -text "Set Start Year" -bg LightGray -fg black\
	  -command [ subst {
                calc_end_year $var1 $var2 $dlg $imagewin $plotwin $top $timeseries $series $bottom $scroll 
                }]

   mstatusbar $dlg.controls.setstartyear \
      "Click Set to set skeleton plot start year."
   
   button $dlg.controls.cancel -text "Cancel" -bg LightGray -fg black\
     -command [ subst {  
               destroy $dlg
              }]
              
   mstatusbar $dlg.controls.cancel \
      "Click Cancel to cancel setting the skeleton plot start year." 
      
   wm protocol $dlg WM_DELETE_WINDOW "$dlg.controls.cancel invoke"  
   
   grid $dlg.controls.setstartyear -row 1 -column 0 
   grid $dlg.controls.cancel -row 1 -column 1
   
}

# Procedure calc_end_year performs the actual reset of the skeleton plot after
# the user has selected a new startyear.
proc calc_end_year { var1 var2 dlg imagewin plotwin top timeseries series bottom scroll } {

   global SkelPlot skelplot

   # Make sure that the endyear is a proper integer number
   set result [regexp {(^\-[0-9]+$)|(^[0-9]+$)} $SkelPlot(startyear) ]   
   if {$result} {
      # Recalculate the global variable skelplot(endyear)
      set skelplot(startyear) $SkelPlot(startyear)
      set skelplot(endyear) [expr $skelplot(startyear) + $SkelPlot(count,rings) - 1]
      set SkelPlot(endyear) $skelplot(endyear)
      
      # Recompute indices, bones for the skeleton plot.   
      compute_indices $var1 $SkelPlot(sample,$series,masterlist) $SkelPlot(sample,$series,serieslist) $series
      compute_bones $var1 $SkelPlot(sample,$series,serieslist)

      # Re-draw the skeleton plot.
      redraw_skelplot $var1 $plotwin $top $timeseries $series $bottom $scroll $imagewin   

      # Destroy this dialog box           
      destroy $dlg
   } else {
       set SkelPlot(endyear) $skelplot(endyear)
   }      
}
