# skelplot_init.tcl - Create and initialize global variables
#                     for Skeleton Plot and Tree Ring Image

# Procedure Index:

# proc skelplot_init { var }
# proc image_init { var }



# -------------------------------------------------------------------


# Initializes SkelPlot global variables and flags
proc skelplot_init {var} {
    upvar #0 $var SkelPlot

    # Variables for computing skeleton plots
    set SkelPlot(abCutoff) 10
    set SkelPlot(diffCutoff) 10
    set SkelPlot(wideCutoff) 0.95

    # Counters used when naming new items
    set SkelPlot(count,rings) 0
    set SkelPlot(count,timeseries) 0
    set SkelPlot(count,skelplots) 0
    
    # Variables used to hold information about the current
    # ring as defined by SkelPlot(count,current)
    set SkelPlot(count,current) 0
    set SkelPlot(count,year) 0
    set SkelPlot(count,leftboundary) ""
    set SkelPlot(count,rightboundary) ""
    set SkelPlot(count,width) 0
    set SkelPlot(count,indexval) 0
    set SkelPlot(count,bonelength) 0
    set SkelPlot(count,typelist) ""
    
    
    # Flag used for displaying removed rings
    set SkelPlot(hideremoved) 0
    
    # Counter used for series and display indices
    set SkelPlot(request) 0
    set SkelPlot(size,actual) 0
    set SkelPlot(size,display) 0
    
    # Fraction offset of the left hand side of the viewable window
    set SkelPlot(leftfrac) 0

    # Scale and size info used when drawing
    set SkelPlot(scale) 10
    set SkelPlot(unitboxes,skelplot) 10
    set SkelPlot(unitboxes,timeseries) 5

    # List of loaded samples
    set SkelPlot(samplelist) {}
    
    # Currently highlighted Ring Boundaries
    set SkelPlot(tags,highlight) {}
    
}

# Initializes Image global variables and flags
proc image_init {var} {
    upvar #0 $var Image

    # Variable used to hold image location and widget information
    # of the form %x %y ?Boundary-?? <typelist> ?Boundary-?? <typelist> ...
    set Image(location) { 0 0 }
}
