/*
##########################################
# Sadie_Camera.c -
#   Set of routines for linking SADIE file I/O routines to tcl/tk.
#
# RCS: $Id: Sadie_Camera.C,v 1.4 2004/10/07 23:00:24 mmunro Exp $ 
# 
# Digital Image Analysis Lab
# Dept. of Electrical and Computer Engineering
# University of Arizona
##########################################
*/

#include <tcl.h>
#include <tk.h>
#include <stdio.h>
#include "cameradevice.h"
#include <sadie.h>
#include "motion.h"
#include "StageMotion.h"

#ifdef __cplusplus
namespace std
{
#endif 

static const int dm = 32;
static const int maxd = dm * 2;

extern Cameradevice* globalcamdev;

 
//int Sadie_Camera_OpenCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
//{
//	printf("initializing the camera...\n");
//	globalcamdev = new Cameradevice(0, -1, 2, 80, 12, 1, 1);
  
//  	if (globalcamdev) {
//  		return TCL_OK;
//  	} else {
//  		return TCL_ERROR;
//  	}
//}



//int Sadie_Camera_CloseCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
//{
//	printf("closing the camera...\n");

//  	if (globalcamdev) {
//        delete globalcamdev;
//        globalcamdev = NULL;
//    }

//  	return TCL_OK;
//}



int Sadie_Camera_AcquireCmd(ClientData client_data, Tcl_Interp* interp, int argc, const char *argv[])
{
	int addrstr;
	IMAGE *theimg=NULL;
	int fromline, toline, incrline, frompixel, topixel, incrpixel;
	char *viewbuf=NULL;
	int j,k,m,n;
	int origheight, origwidth, lines, pixels;
        char   msg[SLEN];

	if( argc <= 6 ) {
		sprintf(msg, "Sadie_Camera_Acquire expects a minimum of 6 arguments");
                Tcl_SetResult(interp, msg, TCL_VOLATILE);
		return TCL_ERROR;
	}

	if (!globalcamdev) {
		sprintf(msg, "camera device not defined");
                Tcl_SetResult(interp, msg, TCL_VOLATILE);
		return TCL_ERROR;
	}


    if( Tcl_GetInt (interp, argv[1], &fromline) != TCL_OK ) {
      return TCL_ERROR;
    }

    if( Tcl_GetInt (interp, argv[2], &toline) != TCL_OK ) {
      return TCL_ERROR;
    }

    if( Tcl_GetInt (interp, argv[3], &incrline) != TCL_OK ) {
      return TCL_ERROR;
    }

    if( Tcl_GetInt (interp, argv[4], &frompixel) != TCL_OK ) {
      return TCL_ERROR;
    }

    if( Tcl_GetInt (interp, argv[5], &topixel) != TCL_OK ) {
      return TCL_ERROR;
    }

    if( Tcl_GetInt (interp, argv[6], &incrpixel) != TCL_OK ) {
      return TCL_ERROR;
    }
    
    origheight = globalcamdev->snap_height();
    origwidth = globalcamdev->snap_width();




    /* check input */
	if (0 > fromline  ||  toline >= origheight) {
        MESSAGE('E'," Image to be sampled must be located completely inside input image in the line dimension.");
        return TCL_ERROR;
    } else if (0 > frompixel  ||  toline >= origwidth) {
        MESSAGE('E'," Image to be sampled must be located completely inside input image in the pixel dimension.");
        return TCL_ERROR;
    } else if (incrline <= 0) {
        MESSAGE('E'," Line subsample increment must be greater than zero.");
        return TCL_ERROR;
    } else if (incrpixel <= 0) {
        MESSAGE('E'," Pixel subsample increment must be greater than zero.");
        return TCL_ERROR;
    } else if (fromline > toline) {
        MESSAGE('E'," Number of lines to be spanned must be greater than zero.");
        return TCL_ERROR;
    } else if (frompixel > topixel) {
        MESSAGE('E'," Number of pixels/line to be spanned must be greater than zero.");
        return TCL_ERROR;
    }


    /* create image of appropriate size */
	lines  = max(1,(1+toline-fromline)/incrline);
	pixels = max(1,(1+topixel-frompixel)/incrpixel);


    //if (!CHECKIMG(theimg)) GETMEM(1,lines,pixels,&theimg);
    
    //printf("lines = %d, pixels = %d\n",lines,pixels);
	//printf("line: %d %d %d\n",fromline,toline,incrline);
	//printf("pixel: %d %d %d\n",frompixel,topixel,incrpixel);
	//printf("orig: %d x %d\n",origwidth,origheight);


	//viewbuf = globalcamdev->acquire();
	
	
    //for (j=0,m=fromline; j<lines && m<origheight; j+=1,m+=incrline) {
    //    for (k=0,n=frompixel; k<pixels && n<origwidth; k+=1,n+=incrpixel) {
    //        theimg->data[0][j][k] = (PIXEL) viewbuf[(m * origwidth) + n];
    //    }
    //}
    
    
	theimg = globalcamdev->acquire_sadie(frompixel,fromline,topixel,toline);
    if (!theimg) return TCL_ERROR;
    
    
    
	addrstr = (int) theimg;
	sprintf(msg, "%x", addrstr);
        Tcl_SetResult(interp, msg, TCL_VOLATILE);
 	

	return TCL_OK;
}


int
Sadie_Camera_BlankCmd (ClientData client_data, Tcl_Interp* interp,
                       int argc, const char *argv[])
{
  IMAGE *nextframe = NULL;
  IMAGE *meanimage = NULL;
  PIXEL *x, *m, residual;
  stagelocationp_t loc, initloc;
  int fromline, toline, frompixel, topixel, nframes;
  int err;
  long int frame, band, line;
  unsigned int pixel;
  char msg[SLEN];
  
  if (argc <= 5)
    {
      snprintf(msg, SLEN,
              "Sadie_Camera_Blank expects a minimum of 5 arguments");
      Tcl_SetResult(interp, msg, TCL_VOLATILE);
      return TCL_ERROR;
    }
  if (!globalcamdev)
    {
      snprintf(msg, SLEN, "camera device not defined");
      Tcl_SetResult(interp, msg, TCL_VOLATILE);
      return TCL_ERROR;
    }
  if (Tcl_GetInt (interp, argv[1], &fromline) != TCL_OK)
    return TCL_ERROR;
  if (Tcl_GetInt (interp, argv[2], &toline) != TCL_OK)
    return TCL_ERROR;
  if (Tcl_GetInt (interp, argv[3], &frompixel) != TCL_OK)
    return TCL_ERROR;
  if (Tcl_GetInt (interp, argv[4], &topixel) != TCL_OK)
    return TCL_ERROR;
  if( Tcl_GetInt (interp, argv[5], &nframes) != TCL_OK)
    return TCL_ERROR;
  if ((0 > fromline) || (toline >= globalcamdev->snap_height()))
    {
      MESSAGE ('E',
               " Image to be sampled must be located completely inside input image in the line dimension.");
      return TCL_ERROR;
    }
  else if ((0 > frompixel) || (toline >= globalcamdev->snap_width()))
    {
      MESSAGE ('E',
               " Image to be sampled must be located completely inside input image in the pixel dimension.");
      return TCL_ERROR;
    }
  else if (fromline > toline)
    {
      MESSAGE ('E',
               " Number of lines to be spanned must be greater than zero.");
      return TCL_ERROR;
    }
  else if (frompixel > topixel)
    {
      MESSAGE ('E',
               " Number of pixels/line to be spanned must be greater than zero.");
      return TCL_ERROR;
    }
  else if (nframes <= 0)
    {
      MESSAGE ('E',
               " Total number of images must be greater than zero.");
      return TCL_ERROR;
    }
  initloc = newstagelocation (2);
  loc = newstagelocation (2);
  if (!initloc || !loc || Stagemotion_Find (initloc))
    {
      MESSAGE ('E', " Could not find stage location.");
      return TCL_ERROR;
    }
  setstagelocation (loc, 0, getstagelocation (initloc, 0));
  setstagelocation (loc, 1, getstagelocation (initloc, 1));
  meanimage = globalcamdev->acquire_sadie (frompixel, fromline, topixel, toline);
  if (!(meanimage && meanimage->nbnd && meanimage->nlin && meanimage->npix))
    {
      MESSAGE ('E', " Initialized the mean with an invalid image.");
      if (meanimage)
        RELIMG (&meanimage);
      return TCL_ERROR;
    }
  for (frame = 2; frame <= nframes; frame++)
    {
      if (!Stagemotion_Active())
        {
          setstagelocation (loc, 0,
                            getstagelocation (loc, 0) + random() % maxd - dm);
          setstagelocation (loc, 1,
                            getstagelocation (loc, 1) + random() % maxd - dm);
          Stagemotion_Position (loc);
        }
      nextframe = globalcamdev->acquire_sadie (frompixel, fromline, topixel, toline);
      if (!nextframe || (nextframe->nbnd != meanimage->nbnd)
          || (nextframe->nlin != meanimage->nlin)
          || (nextframe->npix != meanimage->npix))
        {
          MESSAGE ('E', " The sizes of the acquired and mean images differ.");
          if (meanimage)
            RELIMG (&meanimage);
          if (nextframe)
            RELIMG (&nextframe);
          disposestagelocation (loc);
          disposestagelocation (initloc);
          return TCL_ERROR;
        }
      band = meanimage->nbnd;
      while (band--)
        {
          line = meanimage->nlin;
          while (line--)
            {
              x = nextframe->data[(size_t)band][(size_t)line];
              m = meanimage->data[(size_t)band][(size_t)line];
              for (pixel = meanimage->npix; pixel; pixel--)
                {
                  residual = (*x - *m) / frame;
                  *m += residual;
                  m++;
                  x++;
                }
            }
        }
      if (nextframe)
        RELIMG (&nextframe);
  }
  Stagemotion_Position (initloc);
  Stagemotion_Find (loc);
  if (nextframe)
    RELIMG (&nextframe);
  disposestagelocation (loc);
  disposestagelocation (initloc);
  snprintf(msg, SLEN, "%lx", (unsigned long) meanimage);
  Tcl_SetResult(interp, msg, TCL_VOLATILE);
  return TCL_OK;
}



int Sadie_Camera_SetGainCmd(ClientData client_data, Tcl_Interp* interp, int argc, const char *argv[])
{

	int gain;
        char   msg[SLEN];

	printf("setting the camera gain...\n");
	if( argc <= 1 ) {
		sprintf(msg, "Sadie_Camera_SetGain expects one argument");
                Tcl_SetResult(interp, msg, TCL_VOLATILE);
		return TCL_ERROR;
	}

	if (!globalcamdev) {
		sprintf(msg, "camera device not defined");
                Tcl_SetResult(interp, msg, TCL_VOLATILE);
		return TCL_ERROR;
	}


    if( Tcl_GetInt (interp, argv[1], &gain) != TCL_OK ) {
      return TCL_ERROR;
    }

	if ((gain < -6) || (gain > 18)) {
		sprintf(msg, "A gain of %d is not allowed!",gain);
                Tcl_SetResult(interp, msg, TCL_VOLATILE);
		return TCL_ERROR;
	}


	globalcamdev->set_gain(gain);
	
	return TCL_OK;
}






#ifdef __cplusplus
extern "C" {
#endif 

/*************************************************************************
 *
 * This procedure initializes the the Sadie_Camera procedures when
 * this package is loaded.
 *
 *************************************************************************/
int Sadie_Camera_Init(Tcl_Interp *interp) {
//  Tcl_CreateCommand(interp, "Sadie_Camera_Open", Sadie_Camera_OpenCmd, 
//  	(ClientData) NULL, NULL);
//  Tcl_CreateCommand(interp, "Sadie_Camera_Close", Sadie_Camera_CloseCmd, 
//  	(ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_Camera_Acquire", Sadie_Camera_AcquireCmd, 
  	(ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_Camera_Blank", Sadie_Camera_BlankCmd, 
  	(ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_Camera_SetGain", Sadie_Camera_SetGainCmd, 
  	(ClientData) NULL, NULL);

  return TCL_OK;
}

#ifdef __cplusplus
} // extern "C"
} // namespace std
#endif 
