#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include <tcl.h>
#include <tk.h>
#include "logerr.h"
#include <sadie.h>
#include "proto.h"
#include "sadie_byte.h"
#include "Sadie_Index.h"
#include "trees.h"
#include "mosaic.h"
#include "chain.h"
#include "histogram.h"
#include "trees_profile.h"
#include "trees_widthvec.h"
#if WITH_DMALLOC
#include <dmalloc.h>
#endif /* WITH_DMALLOC */

/*-------------------------------------------------------------*/
/* Global Sadie variables that must be set from within Tcl/Tk  */
/*-------------------------------------------------------------*/
extern 	short   nlev;
extern	short   csize;
extern	double  weight;
extern	double  *count;
extern	PIXEL   gain;
extern	PIXEL   bias;
extern	PIXEL   gmin;
extern	PIXEL   gmax;
extern	PIXEL   thresh;
extern	PIXEL   gbrk[2][4];
extern	PIXEL   *table;
 

extern PIXEL globarray_original[50][100];
extern PIXEL globarray_percent[50][100];
extern PIXEL globarray_normalized[50][100];
extern int globframecount;
extern int globfreqcount;

extern sad_doclistp_t global_doclist;

static const char *measurement_defaults[] =
  {
    "ringwidth",
    "earlywidth",
    "latewidth"
  };

int Sadie_TREES_GetRangeCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
  int imgaddr;
  IMAGE *theimg;
  char   msg[SLEN];

  if( argc <= 1 ) {
    sprintf(msg, "Usage: Sadie_General_GetRange image_address");
    Tcl_SetResult(interp, msg, TCL_VOLATILE);
    return TCL_ERROR;
  }

  sscanf(argv[1], "%x", &imgaddr);
  theimg = (IMAGE *) imgaddr;

  if (!CHECKIMG(theimg)) {
    Tcl_AppendResult(interp, "couldn't identify SADIE image from input string.", NULL);
    return TCL_ERROR;
  }

  RANGE(theimg);

  sprintf(msg, "%f %f", theimg->gmin, theimg->gmax);
  
  Tcl_SetResult(interp, msg, TCL_VOLATILE);

  
  return TCL_OK;
}



int Sadie_TREES_ByteOpenCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
  int addrstr;
  IMAGE_BYTE *byteimg=NULL;
  char   msg[SLEN];

  if( argc <= 1 ) {
    sprintf(msg, "Usage: Sadie_TREES_ByteOpen filename");
    Tcl_SetResult(interp, msg, TCL_VOLATILE);
    return TCL_ERROR;
  }

  BYTEDISK2IMG(argv[1],&byteimg);
  
  addrstr = (int) byteimg;

  sprintf(msg, "%x", addrstr);
  Tcl_SetResult(interp, msg, TCL_VOLATILE);

  return TCL_OK;
}


int Sadie_TREES_ByteSaveCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
  int imgaddr;
  IMAGE_BYTE *byteimg=NULL;
  char* name;
  char   msg[SLEN];
  

  if( argc <= 2 ) {
    sprintf(msg, "Usage: Sadie_TREES_ByteSave image_address filename");
    Tcl_SetResult(interp, msg, TCL_VOLATILE);
    return TCL_ERROR;
  }

  sscanf(argv[1], "%x", &imgaddr);
  byteimg = (IMAGE_BYTE *) imgaddr;

  name = argv[2];


  BYTEIMG2DISK (byteimg,name);
  
  return TCL_OK;
}


int Sadie_TREES_ByteCloseCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
  int imgaddr;
  IMAGE_BYTE *byteimg=NULL;
  char   msg[SLEN];
  

  if( argc <= 1 ) {
    sprintf(msg, "Usage: Sadie_TREES_ByteClose image_address");
    Tcl_SetResult(interp, msg, TCL_VOLATILE);
    return TCL_ERROR;
  }

  sscanf(argv[1], "%x", &imgaddr);
  byteimg = (IMAGE_BYTE *) imgaddr;

  if(CHECKIMG_BYTE(byteimg)) RELIMG_BYTE (&byteimg);
  
  return TCL_OK;
}



int Sadie_TREES_MosaicIndexOpenCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
  int addrstr;
  MOSAIC_INDEX *theindex=NULL;
  char   msg[SLEN];

  if( argc <= 1 ) {
    sprintf(msg, "Usage: Sadie_TREES_MosaicIndexOpen filename");
    Tcl_SetResult(interp, msg, TCL_VOLATILE);
    return TCL_ERROR;
  }

  MOSAIC_DISK2INDEX (argv[1],&theindex);
  
  addrstr = (int) theindex;

  sprintf(msg, "%x", addrstr);
  Tcl_SetResult(interp, msg, TCL_VOLATILE);

  return TCL_OK;
}


int Sadie_TREES_MosaicIndexSaveCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
  int indexaddr;
  MOSAIC_INDEX *theindex=NULL;
  char* name;
  char   msg[SLEN];
  

  if( argc <= 2 ) {
    sprintf(msg, "Usage: Sadie_TREES_MosaicIndexSave image_address filename");
    Tcl_SetResult(interp, msg, TCL_VOLATILE);
    return TCL_ERROR;
  }

  sscanf(argv[1], "%x", &indexaddr);
  theindex = (MOSAIC_INDEX *) indexaddr;
  
  name = argv[2];

  MOSAIC_INDEX2DISK (theindex,name);
  
  return TCL_OK;
}


int Sadie_TREES_MosaicIndexCloseCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
  int indexaddr;
  MOSAIC_INDEX *index=NULL;
  char   msg[SLEN];
  

  if( argc <= 1 ) {
    sprintf(msg, "Usage: Sadie_TREES_MosaicIndexClose index_address");
    Tcl_SetResult(interp, msg, TCL_VOLATILE);
    return TCL_ERROR;
  }

  sscanf(argv[1], "%x", &indexaddr);
  index = (MOSAIC_INDEX *) indexaddr;

  RELIMG_MOSAIC_INDEX (&index);
  
  return TCL_OK;
}
 

int Sadie_TREES_ChainCloseCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
  int  chainaddr;
  char msg[SLEN];
  CHAIN *chainmap = NULL;
  

  if( argc <= 1 ) {
    sprintf(msg, "Usage: Sadie_TREES_ChainClose chainmap_address");
    Tcl_SetResult(interp, msg, TCL_VOLATILE);
    return TCL_ERROR;
  }

  sscanf(argv[1], "%x", &chainaddr);
  chainmap = (CHAIN *) chainaddr;

  RELMEM_CHAIN (chainmap);
  
  return TCL_OK;
}


int Sadie_TREES_CreateMosaicCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
  int i;
  char* imgname[200];
  int width, height, nimg, camh, camw;
  IMAGE_BYTE *outimg=NULL;
  MOSAIC_INDEX *index=NULL;
  int yoffset[200], xoffset[200];
  PIXEL gain[200], bias[200];
  PIXEL fillvalue;
  PIXEL gmin,factor;
  int illum_adjust;
  char* illum_image;
  double tempdouble;
  char   msg[SLEN];

  if( argc <= 7 ) {
    sprintf(msg, "Usage: Sadie_TREES_CreateMosaic nimg height width ...");
    Tcl_SetResult(interp, msg, TCL_VOLATILE);
    return TCL_ERROR;
  }

  if( Tcl_GetInt (interp, argv[1], &nimg) != TCL_OK ) {
      return TCL_ERROR;
  }
  if( Tcl_GetInt (interp, argv[2], &height) != TCL_OK ) {
      return TCL_ERROR;
  }
  if( Tcl_GetInt (interp, argv[3], &width) != TCL_OK ) {
      return TCL_ERROR;
  }
  if( Tcl_GetInt (interp, argv[4], &camh) != TCL_OK ) {
      return TCL_ERROR;
  }
  if( Tcl_GetInt (interp, argv[5], &camw) != TCL_OK ) {
      return TCL_ERROR;
  }
  if( Tcl_GetInt (interp, argv[6], &illum_adjust) != TCL_OK ) {
      return TCL_ERROR;
  }
  illum_image = argv[7];
  if( Tcl_GetDouble (interp, argv[8], &tempdouble) != TCL_OK ) {
      return TCL_ERROR;
  }
  factor = (PIXEL)tempdouble;

  if( Tcl_GetDouble (interp, argv[9], &tempdouble) != TCL_OK ) {
      return TCL_ERROR;
  }
  gmin = (PIXEL)tempdouble;
  

  if( argc <= 9 + (nimg * 5) ) {
    sprintf(msg, "Usage: Sadie_Proto_CreateMosaic nimg height width illum_adjust illum_image factor gmin + (5 * nimg) more entries");
    Tcl_SetResult(interp, msg, TCL_VOLATILE);
    return TCL_ERROR;
  }
  
  for (i=0; i<nimg; i++) {
        imgname[i] = argv[10 + (i*5)];
        
        if( Tcl_GetInt (interp, argv[11 + (i*5)], &xoffset[i]) != TCL_OK ) {
            return TCL_ERROR;
        }
        if( Tcl_GetInt (interp, argv[12 + (i*5)], &yoffset[i]) != TCL_OK ) {
            return TCL_ERROR;
        }
        if( Tcl_GetDouble (interp, argv[13 + (i*5)], &tempdouble) != TCL_OK ) {
            return TCL_ERROR;
        }
        bias[i] = (PIXEL)tempdouble;
        if( Tcl_GetDouble (interp, argv[14 + (i*5)], &tempdouble) != TCL_OK ) {
            return TCL_ERROR;
        }
        gain[i] = (PIXEL)tempdouble;
  }

  fillvalue = (PIXEL) 0.0;
  

  CREATETREESMOSAIC((char **)imgname,illum_adjust,illum_image,nimg,height,width,camh,camw,(int *)xoffset,(int *)yoffset,(PIXEL *)bias,(PIXEL *)gain,gmin,factor,fillvalue,(IMAGE_BYTE **)&outimg,(MOSAIC_INDEX **)&index);


  sprintf(msg, "%x %x", (int)outimg, (int)index);
  Tcl_SetResult(interp, msg, TCL_VOLATILE);


  return TCL_OK;
}



int Sadie_TREES_ByteMosaicResampCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{  
  int inimgaddr, outimgaddr, indexaddr;
  int lines, pix, incrlines, incrpix;
  IMAGE_BYTE *inimg=NULL;
  MOSAIC_INDEX *index=NULL;
  IMAGE *outimg=NULL;
  char   msg[SLEN];

  if( argc <= 6 ) {
    return TCL_ERROR;
  }
 
 
  sscanf(argv[1], "%x", &inimgaddr);
  inimg = (IMAGE_BYTE *) inimgaddr;

  sscanf(argv[2], "%x", &indexaddr);
  index = (MOSAIC_INDEX *) indexaddr;

  if( Tcl_GetInt (interp, argv[3], &lines) != TCL_OK ) {
    return TCL_ERROR;
  }

  if( Tcl_GetInt (interp, argv[4], &pix) != TCL_OK ) {
    return TCL_ERROR;
  }

  if( Tcl_GetInt (interp, argv[5], &incrlines) != TCL_OK ) {
    return TCL_ERROR;
  }

  if( Tcl_GetInt (interp, argv[6], &incrpix) != TCL_OK ) {
    return TCL_ERROR;
  }



  MOSAIC_BYTE_RESAMPL (inimg, index, lines, pix, incrlines, incrpix, &outimg);

  outimgaddr = (int) outimg;
  sprintf(msg, "%x", outimgaddr);
  Tcl_SetResult(interp, msg, TCL_VOLATILE);
  
  return TCL_OK;

}  


int Sadie_TREES_MosaicImportCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{  
  int outimgaddr, indexaddr;
  int jbgn, jend, jinc, kbgn, kend, kinc;
  MOSAIC_INDEX *index=NULL;
  IMAGE *outimg=NULL;
  char   msg[SLEN];

  
  if( argc <= 8 ) {
    return TCL_ERROR;
  }
 
 
  sscanf(argv[1], "%x", &indexaddr);
  index = (MOSAIC_INDEX *) indexaddr;

  if( Tcl_GetInt (interp, argv[3], &jbgn) != TCL_OK ) {
    return TCL_ERROR;
  }

  if( Tcl_GetInt (interp, argv[4], &jend) != TCL_OK ) {
    return TCL_ERROR;
  }

  if( Tcl_GetInt (interp, argv[5], &jinc) != TCL_OK ) {
    return TCL_ERROR;
  }

  if( Tcl_GetInt (interp, argv[6], &kbgn) != TCL_OK ) {
    return TCL_ERROR;
  }

  if( Tcl_GetInt (interp, argv[7], &kend) != TCL_OK ) {
    return TCL_ERROR;
  }

  if( Tcl_GetInt (interp, argv[8], &kinc) != TCL_OK ) {
    return TCL_ERROR;
  }



  MOSAIC_IMPORT (index, argv[2], jbgn, kbgn, jinc, kinc, jend, kend, &outimg);

  outimgaddr = (int) outimg;
  sprintf(msg, "%x", outimgaddr);
  Tcl_SetResult(interp, msg, TCL_VOLATILE);
  
  return TCL_OK;

}  


int Sadie_TREES_ByteMosaicExportCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
  int imgaddr;
  IMAGE_BYTE *theimg=NULL;
  char* name;
  char   msg[SLEN];

  
  if( argc <= 2 ) {
    sprintf(msg, "Usage: Sadie_TREES_ByteMosaicExport image_address filename");
    Tcl_SetResult(interp, msg, TCL_VOLATILE);
    return TCL_ERROR;
  }

  sscanf(argv[1], "%x", &imgaddr);
  theimg = (IMAGE_BYTE *) imgaddr;

  name = argv[2];


  MOSAIC_BYTE_EXPORT (theimg,name);
  
  return TCL_OK;
}



int Sadie_TREES_Chain2DiskCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
  int addrstr;
  CHAIN *chainmap = NULL;
  char *filename;
  char   msg[SLEN];

  if( argc <= 2 ) {
    sprintf(msg, "Usage: Sadie_TREES_Chain2Disk chainmap filename");
    Tcl_SetResult(interp, msg, TCL_VOLATILE);
    return TCL_ERROR;
  }

  sscanf(argv[1], "%x", &addrstr);
  chainmap = (CHAIN *) addrstr;

  filename = argv[2];

  CHAIN2DISK(chainmap, filename);

  return TCL_OK;
}



int Sadie_TREES_Disk2ChainCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
  CHAIN *chainmap = NULL;
  char   msg[SLEN];

  if( argc <= 1 ) {
    sprintf(msg, "Usage: Sadie_TREES_Disk2Chain filename");
    Tcl_SetResult(interp, msg, TCL_VOLATILE);
    return TCL_ERROR;
  }

  DISK2CHAIN(argv[1], &chainmap);
  
  sprintf(msg, "%x", (int) chainmap);
  Tcl_SetResult(interp, msg, TCL_VOLATILE);

  return TCL_OK;
}



int Sadie_TREES_ScaleChainCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
  int addrstr;
  int scale;
  CHAIN *chainmap = NULL;
  CHAIN *scaledChain = NULL;
  char   msg[SLEN];

  if( argc <= 2 ) {
    sprintf(msg, "Usage: Sadie_TREES_ScaleChain chainmap scale");
    Tcl_SetResult(interp, msg, TCL_VOLATILE);
    return TCL_ERROR;
  }

  sscanf(argv[1], "%x", &addrstr);
  chainmap = (CHAIN *) addrstr;

  if( Tcl_GetInt (interp, argv[2], &scale) != TCL_OK ) {
    return TCL_ERROR;
  }

  SCALE_CHAIN(chainmap, scale, &scaledChain);
  
  sprintf(msg, "%x", (int) scaledChain);
  Tcl_SetResult(interp, msg, TCL_VOLATILE);

  return TCL_OK;
}



int Sadie_TREES_DisplayRingsCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
    int prevx,prevy,currx,curry;
    int addrstr;
    CHAIN *chainmap = NULL;
    CHAIN_NODE *tempCnode = NULL;
    LIST_NODE  *tempLnode = NULL;
    char   msg[SLEN];
    char *canvas = NULL;
    char ringcolor[50], ringtag[50];
    
    if( argc <= 2 ) {
	sprintf(msg, "Usage: Sadie_TREES_DisplayRings chainmap canvasname");
	Tcl_SetResult(interp, msg, TCL_VOLATILE);
	return TCL_ERROR;
    }
    
    sscanf(argv[1], "%x", &addrstr);
    chainmap = (CHAIN *) addrstr;
    
    canvas = argv[2];
    
    tempLnode = chainmap->list;
    while(tempLnode != NULL) {

	prevx = tempLnode->startk;
	prevy = tempLnode->startj;
	currx = prevx;
	curry = prevy;
	
	switch (tempLnode->ring_type) {
	case NORMAL :
	    sprintf(ringcolor, "%s", "$boundarycolor(normal)");
	    sprintf(ringtag, "\"Boundary-%d normal\"", tempLnode->ring_id);
	    break;

	case ADDED :
	    sprintf(ringcolor, "%s", "$boundarycolor(added)");
	    sprintf(ringtag, "\"Boundary-%d added\"", tempLnode->ring_id);
	    break;
	    
	case REMOVED :
	    sprintf(ringcolor, "%s", "$boundarycolor(removed)");
	    sprintf(ringtag, "\"Boundary-%d removed\"", tempLnode->ring_id);
	    break;
	}
	
	tempCnode = tempLnode->chain;
	while(tempCnode != NULL) {
	    
	    CHAIN_COMPUTE_COORDINATES(currx, curry, tempCnode->code, &currx, &curry);
	    sprintf(msg, "%s create line %d %d %d %d -width 1 -fill %s -tag %s", canvas, prevx, prevy, currx, curry, ringcolor, ringtag); 
	    
	    if( Tcl_Eval(interp, msg) != TCL_OK ) {
		return TCL_ERROR;
	    }
	    
	    prevx = currx;
	    prevy = curry;
	    tempCnode = (CHAIN_NODE *)tempCnode->next;
	}
	tempLnode = tempLnode->next;
    }
    
    return TCL_OK;
}



/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1999 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This procedure provides an interface to the functions SPLINE             */
/*   and DIVSER from Tcl/Tk in order to remove trend from tree ring           */
/*   measurements.  It expects a tcl global array                             */
/*   with these indices to exist:                                             */
/*      array(count)              --  int                                     */
/*      array([0-count-1],width)  --  double                                  */
/*                                                                            */
/*   It will create the following indices:                                    */
/*      array([0-count-1],indexval)  --  double                               */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
int Sadie_TREES_ComputeIndicesCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
    Tcl_Obj* tclobj=NULL;
    Tcl_Obj* tclarrayname = NULL;
    Tcl_Obj* tclindexname = NULL;
    char msg[SLEN];
    char* array=NULL;
    int i, N;
    float *Z, *ZF, *RZ, ZSP, ZPV;
    double tempval;

    if( argc != 2 ) {
	Tcl_AppendResult(interp,"wrong # args: should be \"",
	argv[0], " arrayname\"", (char *) NULL);
	return TCL_ERROR;
    }
    array=argv[1];
		

    /* Read the input integer array(count) */
    tclarrayname = Tcl_NewStringObj(array,-1);
    tclindexname = Tcl_NewStringObj("count",-1);
    if ((tclobj = Tcl_ObjGetVar2(interp,  tclarrayname , tclindexname , TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG))) {
	if (Tcl_GetIntFromObj(interp,tclobj,&N) == TCL_ERROR) return TCL_ERROR;
    } else {
	return TCL_ERROR;
    }
    Tcl_DecrRefCount(tclarrayname);
    Tcl_DecrRefCount(tclindexname);

    ZSP = 12.0;
    ZPV = 0.5;
    Z = (float *)malloc(N*sizeof(float));
    ZF = (float *)malloc(N*sizeof(float));
    RZ = (float *)malloc(N*sizeof(float));

    /* Read input width array([0-count-1],width) */
    for (i=0; i<N; i++) {
	/* Read the input width values */
	tclarrayname = Tcl_NewStringObj(array,-1);
	sprintf(msg,"%d,width",i);
	tclindexname = Tcl_NewStringObj(msg,-1);
	if ((tclobj = Tcl_ObjGetVar2(interp,  tclarrayname , tclindexname , TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG))) {
	    if (Tcl_GetDoubleFromObj(interp,tclobj,&tempval) == TCL_ERROR) return TCL_ERROR;
	} else {
	    return TCL_ERROR;
	}
	Tcl_DecrRefCount(tclarrayname);
	Tcl_DecrRefCount(tclindexname);

	Z[i] = (float) tempval;
	ZF[i] = 0.0;
    }

    SPLINE(N, Z, ZF, ZSP, ZPV);
    DIVSER(N, Z, ZF, RZ);
    for(i = 0; i < N; i++)
    {
	tempval = (double) RZ[i];
	
	/* Set the output index values */
	tclobj = Tcl_NewDoubleObj(tempval);
	tclarrayname = Tcl_NewStringObj(array,-1);
	sprintf(msg,"%d,indexval",i);
	tclindexname = Tcl_NewStringObj(msg,-1);
	Tcl_ObjSetVar2(interp,  tclarrayname , tclindexname , tclobj, TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG);
	Tcl_DecrRefCount(tclarrayname);
	Tcl_DecrRefCount(tclindexname);
    }

    if (Z) free(Z);
    if (ZF) free(ZF);
    if (RZ) free(RZ);
	    
    return TCL_OK;
}



int Sadie_TREES_DisplayPartRingsCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
    int X1, Y1, X2, Y2;
    int prevx,prevy,currx,curry;
    int addrstr;
    CHAIN *chainmap = NULL;
    CHAIN_NODE *tempCnode = NULL;
    LIST_NODE  *tempLnode = NULL;
    char   msg[500];
    char *canvas = NULL;
    char ringcolor[50], ringtag[50];
    
    if( argc <= 6 ) {
	sprintf(msg, "Usage: Sadie_TREES_DisplayPartRings chainmap X1 Y1 X2 Y2 canvasname");
	Tcl_SetResult(interp, msg, TCL_VOLATILE);
	return TCL_ERROR;
    }
    
    sscanf(argv[1], "%x", &addrstr);
    chainmap = (CHAIN *) addrstr;
    
    if( Tcl_GetInt (interp, argv[2], &X1) != TCL_OK ) {
	return TCL_ERROR;
    }
    if( Tcl_GetInt (interp, argv[3], &Y1) != TCL_OK ) {
	return TCL_ERROR;
    }
    if( Tcl_GetInt (interp, argv[4], &X2) != TCL_OK ) {
	return TCL_ERROR;
    }
    if( Tcl_GetInt (interp, argv[5], &Y2) != TCL_OK ) {
	return TCL_ERROR;
    }
   
    canvas = argv[6];
    
    tempLnode = chainmap->list;
    while(tempLnode != NULL && (int)tempLnode->startk < X1 - 100) 
	tempLnode = tempLnode->next;

    
    while(tempLnode != NULL && tempLnode->startk < X2 + 100) {
	prevx = tempLnode->startk;
	prevy = tempLnode->startj;
	currx = prevx;
	curry = prevy;

	switch (tempLnode->ring_type) {
	case NORMAL :
	    sprintf(ringcolor, "%s", "$boundarycolor(normal)");
	    sprintf(ringtag, "\"ring Boundary-%d normal\"", tempLnode->ring_id);
	    break;

	case ADDED :
	    sprintf(ringcolor, "%s", "$boundarycolor(added)");
	    sprintf(ringtag, "\"ring Boundary-%d added\"", tempLnode->ring_id);
	    break;
	    
	case REMOVED :
	    sprintf(ringcolor, "%s", "$boundarycolor(removed)");
	    sprintf(ringtag, "\"ring Boundary-%d removed\"", tempLnode->ring_id);
	    break;
	}
	
	tempCnode = tempLnode->chain;
	while(tempCnode != NULL) {
	    
	    CHAIN_COMPUTE_COORDINATES(currx, curry, tempCnode->code, &currx, &curry);
		
	    if (curry >= Y1) {
		sprintf(msg, "%s create line %d %d %d %d -width 1 -fill %s -tag %s", canvas, prevx - X1, prevy - Y1, currx - X1, curry - Y1, ringcolor, ringtag);
	    
	    
		if( Tcl_Eval(interp, msg) != TCL_OK ) {
		    return TCL_ERROR;
		}
	    }

	    prevx = currx;
	    prevy = curry;
	    tempCnode = (CHAIN_NODE *)tempCnode->next;
	    
	    if (curry > Y2)
		break; 
	}
	tempLnode = tempLnode->next;
    }
    
    return TCL_OK;
}


int Sadie_TREES_UpdateROICmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
    uint32_t left, right, top, bottom;
    int hstart, hend, vtop, vbottom, zoom,tcl_flag;
    int i, X[4], Y[4];
    int addrstr;
    MOSAIC_INDEX *index = NULL;
    char   msg[SLEN];
    
    if( argc <= 11 ) {
	sprintf(msg, "Usage: Sadie_TREES_UpdateROI index <current zoom factor> P1 P2 P4 P5");
	Tcl_SetResult(interp, msg, TCL_VOLATILE);
	return TCL_ERROR;
    }

    sscanf(argv[1], "%x", &addrstr);
    index = (MOSAIC_INDEX *) addrstr;
    
    if( Tcl_GetInt (interp, argv[2], &zoom) != TCL_OK ) {
	return TCL_ERROR;
    }
    if( Tcl_GetInt (interp, argv[3], &X[0]) != TCL_OK ) {
	return TCL_ERROR;
    }
    if( Tcl_GetInt (interp, argv[4], &Y[0]) != TCL_OK ) {
	return TCL_ERROR;
    }
    if( Tcl_GetInt (interp, argv[5], &X[1]) != TCL_OK ) {
	return TCL_ERROR;
    }
    if( Tcl_GetInt (interp, argv[6], &Y[1]) != TCL_OK ) {
	return TCL_ERROR;
    }
    if( Tcl_GetInt (interp, argv[7], &X[2]) != TCL_OK ) {
	return TCL_ERROR;
    }
    if( Tcl_GetInt (interp, argv[8], &Y[2]) != TCL_OK ) {
	return TCL_ERROR;
    }
    if( Tcl_GetInt (interp, argv[9], &X[3]) != TCL_OK ) {
	return TCL_ERROR;
    }
    if( Tcl_GetInt (interp, argv[10], &Y[3]) != TCL_OK ) {
	return TCL_ERROR;
    }

    if( Tcl_GetInt (interp, argv[11], &tcl_flag) != TCL_OK ) {
	return TCL_ERROR;
    }

    hstart = index->npix -1;
    hend = 0;
    vtop = index->nlin -1;
    vbottom = 0;
    left = 0;
    right = 0;
    top = 0;
    bottom= 0;
    /* Compute coordinates at full resolution and identify extremal points */
    for(i = 0; i < 4; i++) {
	X[i] = X[i] * zoom;
	Y[i] = Y[i] * zoom;
	if (X[i] == hstart) {
	    left = (Y[i] < Y[left]? i: left);
	} else if (X[i] < hstart) {
	    hstart = X[i];
	    left = i;
	}
	if (X[i] == hend) {
	    right = (Y[i] > Y[right]? i: right);
	} else if (X[i] > hend) {
	    hend = X[i];
	    right = i;
	}
	if (Y[i] == vtop) {
	    top = (X[i] > X[top]? i: top);
	} else if (Y[i] < vtop) {
	    vtop = Y[i];
	    top = i;
	}
	if (Y[i] == vbottom) {
	    bottom = (X[i] < X[bottom]? i: bottom);
	} else if (Y[i] > vbottom) {
	    vbottom = Y[i];
	    bottom = i;
	}
    }
  if (tcl_flag == 1)
  {
    /* Update hstart, hend in index structure */
    index->hstart = hstart; 
    index->hend = hend; 

    /* Update the extremal points in the selected region-of-interest */
    index->roi->left[0] = Y[left];
    index->roi->left[1] = X[left];
    
    index->roi->right[0] = Y[right];
    index->roi->right[1] = X[right];
    
    index->roi->top[0] = Y[top];
    index->roi->top[1] = X[top];
    
    index->roi->bottom[0] = Y[bottom];
    index->roi->bottom[1] = X[bottom];

    printf("\n>> Sadie >> top(%d,%d), bottom(%d,%d), left(%d,%d), right(%d,%d)\n", Y[top],X[top],Y[bottom],X[bottom],Y[left],X[left],Y[right],X[right]);
    
    /* Compute vstart going top->left */
    COMPUTEVSTART(index, X[top], Y[top], X[left], Y[left]);
    
    /* Compute vstart going top->right */
    COMPUTEVSTART(index, X[top], Y[top], X[right], Y[right]);
    
    /* Compute vend going bottom->right */
    COMPUTEVEND(index, X[bottom], Y[bottom], X[right], Y[right]);
    
    /* Compute vend going bottom->left */
    COMPUTEVEND(index, X[bottom], Y[bottom], X[left], Y[left]);

    /* The Bresh algorithm possibly goofs up on the last co-ordinate
       pair it returns. The following code will fix the problem. */
    index->vstart[hstart] = Y[left];
    index->vstart[hend] = Y[right];
    index->vend[hend] = Y[right];
    index->vend[hstart] = Y[left];

    return TCL_OK;
   }
  else if (tcl_flag == 0)
    {
     printf("\n top= %d,%d, bottom=%d,%d \n",hstart,vtop,hend,vbottom );
     /* return the ROI coordinate to tcl layer */
 
  sprintf(msg,"global roi_minx\nset roi_minx %d",hstart);
  if( Tcl_Eval(interp, msg) != TCL_OK ) {
      return TCL_ERROR;
       }
  sprintf(msg,"global roi_miny\nset roi_miny %d",vtop);
  if( Tcl_Eval(interp, msg) != TCL_OK ) {
      return TCL_ERROR;
       }
  sprintf(msg,"global roi_maxx\nset roi_maxx %d",hend);
  if( Tcl_Eval(interp, msg) != TCL_OK ) {
      return TCL_ERROR;
       }
  sprintf(msg,"global roi_maxy\nset roi_maxy %d",vbottom);
  if( Tcl_Eval(interp, msg) != TCL_OK ) {
      return TCL_ERROR;
       } 
     return TCL_OK;
    }
}

int Sadie_TREES_Chain_RingWidthsCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
  int i, j, npix;
  int addrstr;
  IMAGE_BYTE *mosaicimg=NULL;
  MOSAIC_INDEX *index=NULL;
  CHAIN *chainmap=NULL;
  PROFILE_LIST profiles=NULL;
  PROFILE_LIST ringprofile;
  PROFILE_LIST current_diffprofile;
  BOUNDBOX_LIST bound_box;
  BOUNDBOX_LIST current_bound_box;

  char   msg[SLEN];
  char *widthlist=NULL;
  char *profilelist=NULL;
  char *diff_profilelist=NULL;
  char *earlywidthlist=NULL;  
  char *latewidthlist=NULL;
  char *boundboxlist;
  widthvecp_t ringwidths=NULL;
  
  PROFILE_LIST diffprofiles;
  widthvecp_t lateringwidth=NULL;
  widthvecp_t earlyringwidth=NULL;
  
  if( argc <=  10) {
    sprintf(msg, "Usage: Sadie_TREES_RingWidths document mosaic index chainmap widthlist profilelist earlywoodlist latewoodlist boundbox");
    Tcl_SetResult(interp, msg, TCL_VOLATILE);
    return TCL_ERROR;
  }

  sscanf(argv[2], "%x", &addrstr);
  mosaicimg = (IMAGE_BYTE *) addrstr;

  sscanf(argv[3], "%x", &addrstr);
  index = (MOSAIC_INDEX *) addrstr;

  sscanf(argv[4], "%x", &addrstr);
  chainmap = (CHAIN *) addrstr;

  widthlist = argv[5];

  profilelist = argv[6];
  
  diff_profilelist = argv[7];
  
  earlywidthlist = argv[8];

  latewidthlist = argv[9];
  
  boundboxlist = argv[10];
  
  
  ringwidths = malloc (sizeof (struct Widthvec));
  if (ringwidths == NULL)
    logerr_crit (LOGERR_FUNC, "allocating memory for ring widths");
  earlyringwidth = malloc (sizeof (struct Widthvec));
  if (earlyringwidth == NULL)
    logerr_crit (LOGERR_FUNC, "allocating memory for earlywood widths");
  lateringwidth = malloc (sizeof (struct Widthvec));
  if (lateringwidth == NULL)
    logerr_crit (LOGERR_FUNC, "allocating memory for latewood widths");
  CHAIN_RINGWIDTHS(mosaicimg, index, chainmap, &profiles, &ringwidths->len, &ringwidths->val, &bound_box);

  First_diff_profiles(&profiles, &diffprofiles, ringwidths, earlyringwidth, lateringwidth);

  sad_put_doc_property (global_doclist, argv[1], "ringwidth", widthvec_destruct, ringwidths);
  sad_put_doc_property (global_doclist, argv[1], "earlywidth", widthvec_destruct, earlyringwidth);
  sad_put_doc_property (global_doclist, argv[1], "latewidth", widthvec_destruct, lateringwidth);
  sprintf(msg,"global %s\nset %s {}",widthlist,widthlist);
  if( Tcl_Eval(interp, msg) != TCL_OK ) {
      return TCL_ERROR;
  }
  sprintf(msg,"global %s\nset %s {}",profilelist,profilelist);
  if( Tcl_Eval(interp, msg) != TCL_OK ) {
      return TCL_ERROR;
  }
  
  sprintf(msg,"global %s\nset %s {}",diff_profilelist,diff_profilelist);
  if( Tcl_Eval(interp, msg) != TCL_OK ) {
      return TCL_ERROR;
  }
  
  sprintf(msg,"global %s\nset %s {}",earlywidthlist,earlywidthlist);
  if( Tcl_Eval(interp, msg) != TCL_OK ) {
      return TCL_ERROR;
  }
 
  sprintf(msg,"global %s\nset %s {}",latewidthlist,latewidthlist);
  if( Tcl_Eval(interp, msg) != TCL_OK ) {
      return TCL_ERROR;
  } 
  sprintf(msg,"global %s\nset %s {}",boundboxlist,boundboxlist);
  if( Tcl_Eval(interp, msg) != TCL_OK ) {
      return TCL_ERROR;
   }
 
  current_bound_box = bound_box;
  ringprofile = profiles;
  current_diffprofile = diffprofiles;
  for (i=0; i<ringwidths->len; i++) {
    sprintf(msg,"lappend %s %f",widthlist,ringwidths->val[i]);
      if( Tcl_Eval(interp, msg) != TCL_OK ) {
	  return TCL_ERROR;
	  }
      
    sprintf(msg,"lappend %s %f",earlywidthlist,earlyringwidth->val[i]);
      if( Tcl_Eval(interp, msg) != TCL_OK ) {
	  return TCL_ERROR;
	  }
    sprintf(msg,"lappend %s %f",latewidthlist,lateringwidth->val[i]);
      if( Tcl_Eval(interp, msg) != TCL_OK ) {
	  return TCL_ERROR;
	  } 
 
     sprintf(msg,"lappend %s %f",boundboxlist,current_bound_box->minx);
      if( Tcl_Eval(interp, msg) != TCL_OK ) {
	  return TCL_ERROR;
	  }      
     sprintf(msg,"lappend %s %f",boundboxlist,current_bound_box->miny);
      if( Tcl_Eval(interp, msg) != TCL_OK ) {
	  return TCL_ERROR;
	  }
     sprintf(msg,"lappend %s %f",boundboxlist,current_bound_box->maxx);
      if( Tcl_Eval(interp, msg) != TCL_OK ) {
	  return TCL_ERROR;
      }
     sprintf(msg,"lappend %s %f",boundboxlist,current_bound_box->maxy);
      if( Tcl_Eval(interp, msg) != TCL_OK ) {
	  return TCL_ERROR;
	  }


      sprintf(msg,"set proftemp {}\n");
      if( Tcl_Eval(interp, msg) != TCL_OK ) {
	  return TCL_ERROR;
	  }

      sprintf(msg,"set diff_proftemp {}\n");
      if( Tcl_Eval(interp, msg) != TCL_OK ) {
	  return TCL_ERROR;
      }

      if ((ringwidths->val[i] < MAXPROFILE) && (ringprofile != NULL)) {
        npix = ringprofile->len;
        for (j = 0; j < npix; j++) {
	   sprintf(msg,"lappend proftemp %f",ringprofile->val[j]);
          if( Tcl_Eval(interp, msg) != TCL_OK ) {
            return TCL_ERROR;
	    }
           
          sprintf(msg,"lappend diff_proftemp %f",current_diffprofile->val[j]);
          if( Tcl_Eval(interp, msg) != TCL_OK ) {
            return TCL_ERROR;
          }           

         }
      }
      if (ringprofile) { ringprofile = ringprofile->next; current_diffprofile =  current_diffprofile->next; 
                         current_bound_box = current_bound_box->next; 
                       }
      sprintf(msg,"lappend %s $proftemp",profilelist);
      if( Tcl_Eval(interp, msg) != TCL_OK ) {
	  return TCL_ERROR;
	  }

      sprintf(msg,"lappend %s $diff_proftemp",diff_profilelist);
      if( Tcl_Eval(interp, msg) != TCL_OK ) {
	  return TCL_ERROR;
      }

  }
  if (current_bound_box != NULL)
    {
      sprintf(msg,"lappend %s %f",boundboxlist,current_bound_box->minx);
      if( Tcl_Eval(interp, msg) != TCL_OK )
        return TCL_ERROR;
      sprintf(msg,"lappend %s %f",boundboxlist,current_bound_box->miny);
      if( Tcl_Eval(interp, msg) != TCL_OK )
        return TCL_ERROR;
      sprintf(msg,"lappend %s %f",boundboxlist,current_bound_box->maxx);
      if( Tcl_Eval(interp, msg) != TCL_OK )
        return TCL_ERROR;
      sprintf(msg,"lappend %s %f",boundboxlist,current_bound_box->maxy);
      if( Tcl_Eval(interp, msg) != TCL_OK )
        return TCL_ERROR;
    }

  /*  if (ringwidths) { free(ringwidths);free(lateringwidth);free(earlyringwidth);} */
  if (profiles) { demolish_profiles(profiles); demolish_profiles(diffprofiles); demolish_boundbox(bound_box);}
  return TCL_OK;
}


int Sadie_TREES_QueryRingTypeCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
    int ringnum;
    int addrstr;
    CHAIN *chainmap = NULL;
    LIST_NODE *tempLnode;
    char   msg[SLEN];
    
    if( argc <= 2 ) {
	sprintf(msg, "Usage: Sadie_TREES_QueryRingType chainmap <ring_id of ring to query>");
	Tcl_SetResult(interp, msg, TCL_VOLATILE);
	return TCL_ERROR;
    }
    
    sscanf(argv[1], "%x", &addrstr);
    chainmap = (CHAIN *) addrstr;
    
    if( Tcl_GetInt (interp, argv[2], &ringnum) != TCL_OK ) {
	return TCL_ERROR;
    }

    tempLnode = chainmap->list;
    while (tempLnode != NULL && tempLnode->ring_id != ringnum) 
	tempLnode = tempLnode->next;


    if (tempLnode == NULL) {
	sprintf(msg, "QueryRingType >> Ring %d not found", ringnum);
	Tcl_SetResult(interp, msg, TCL_VOLATILE);
	return TCL_ERROR;
    }

    switch (tempLnode->ring_type) {
    case NORMAL :

    case ADDED :
	
    case REMOVED :
	sprintf(msg, "%d", (int)tempLnode->ring_type);
	Tcl_SetResult(interp, msg, TCL_VOLATILE);
	break;

    default :
	sprintf(msg, "QueryRingType >> Ring %d - unknown ring type", ringnum);
	Tcl_SetResult(interp, msg, TCL_VOLATILE);
	return TCL_ERROR;
    }
    
    return TCL_OK;
}



int Sadie_TREES_QueryBoundaryIdCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
    int count, index;
    int addrstr;
    CHAIN *chainmap = NULL;
    LIST_NODE *tempLnode;
    char   msg[SLEN];
    
    if( argc <= 2 ) {
	sprintf(msg, "Usage: Sadie_TREES_QueryBoundaryId chainmap index");
	Tcl_SetResult(interp, msg, TCL_VOLATILE);
	return TCL_ERROR;
    }
    
    sscanf(argv[1], "%x", &addrstr);
    chainmap = (CHAIN *) addrstr;
    
    if( Tcl_GetInt (interp, argv[2], &index) != TCL_OK ) {
	return TCL_ERROR;
    }

    if (index < 0) {
	sprintf(msg, "QueryBoundaryId >> Index is Negative");
	Tcl_SetResult(interp, msg, TCL_VOLATILE);
	return TCL_ERROR;
    }
    
    count = 0;
    tempLnode = chainmap->list;
    while (tempLnode != NULL && count++ != index) 
	tempLnode = tempLnode->next;


    if (tempLnode == NULL) {
	sprintf(msg, "QueryBoundaryId >> There are fewer than  %d rings", index+1);
	Tcl_SetResult(interp, msg, TCL_VOLATILE);
	return TCL_ERROR;
    }

    sprintf(msg, "%d", (int)tempLnode->ring_id);
    Tcl_SetResult(interp, msg, TCL_VOLATILE);

    return TCL_OK;
}


int Sadie_TREES_Mosaic2MosaicResamplCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{  
  int inimgaddr, outimgaddr, indexaddr;
  int lines, pix, incrlines, incrpix;
  IMAGE_BYTE *inimg=NULL;
  MOSAIC_INDEX *index=NULL;
  IMAGE_BYTE *outimg=NULL;
  char   msg[SLEN];

  if( argc <= 6 ) {
    return TCL_ERROR;
  }
 
 
  sscanf(argv[1], "%x", &inimgaddr);
  inimg = (IMAGE_BYTE *) inimgaddr;

  sscanf(argv[2], "%x", &indexaddr);
  index = (MOSAIC_INDEX *) indexaddr;

  if( Tcl_GetInt (interp, argv[3], &lines) != TCL_OK ) {
    return TCL_ERROR;
  }

  if( Tcl_GetInt (interp, argv[4], &pix) != TCL_OK ) {
    return TCL_ERROR;
  }

  if( Tcl_GetInt (interp, argv[5], &incrlines) != TCL_OK ) {
    return TCL_ERROR;
  }

  if( Tcl_GetInt (interp, argv[6], &incrpix) != TCL_OK ) {
    return TCL_ERROR;
  }



  MOSAIC2MOSAIC_RESAMPL (inimg, index, lines, pix, incrlines, incrpix, &outimg);

  outimgaddr = (int) outimg;
  sprintf(msg, "%x", outimgaddr);
  Tcl_SetResult(interp, msg, TCL_VOLATILE);
  
  return TCL_OK;

}  

int Sadie_TREES_IndexResamplCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{  
  int indexaddr, outindaddr;
  int scale;
  MOSAIC_INDEX *index=NULL;
  MOSAIC_INDEX *scaledindex=NULL;
  char   msg[SLEN];

  if( argc <= 2 ) {
    return TCL_ERROR;
  }
 
 
  sscanf(argv[1], "%x", &indexaddr);
  index = (MOSAIC_INDEX *) indexaddr;

  if( Tcl_GetInt (interp, argv[2], &scale) != TCL_OK ) {
    return TCL_ERROR;
  }

  INDEX_RESAMPL (index, scale, &scaledindex);

  outindaddr = (int) scaledindex;
  sprintf(msg, "%x", outindaddr);
  Tcl_SetResult(interp, msg, TCL_VOLATILE);
  
  return TCL_OK;

}  

static int
trees_tcl_glue_exporter (void *ctx, char *buf)
{
  int err = 0;
  Tcl_Channel channel;

  channel = ctx;
  err = Tcl_WriteChars (channel, buf, -1);
  if (err >= 0)
    return 0;
  else
    {
      err = Tcl_GetErrno();
      return err;
    }
}

int
Sadie_TREES_ExportRwlCmd(ClientData client_data, Tcl_Interp* interp,
                         int argc, char *argv[])
{  
  Tcl_Channel channel;
  int mode, yr, status;

  if (argc < 6)
    return TCL_ERROR;
  channel = Tcl_GetChannel (interp, argv[1], &mode);
  if ((channel == NULL) || !(mode & TCL_WRITABLE))
    return TCL_ERROR;
  status = Tcl_GetInt (interp, argv[2], &yr);
  if (status != TCL_OK)
    return status;
  trees_rwl_export (global_doclist, argv[3],
                    trees_tcl_glue_exporter, (void *) channel,
                    yr, argv[4], argv[5]);
  return TCL_OK;
}  

int
Sadie_TREES_ExportCsvCmd(ClientData client_data, Tcl_Interp* interp,
                         int argc, char *argv[])
{  
  Tcl_Channel channel;
  int mode, yr, status;

  if (argc < 4)
    return TCL_ERROR;
  channel = Tcl_GetChannel (interp, argv[1], &mode);
  if ((channel == NULL) || !(mode & TCL_WRITABLE))
    return TCL_ERROR;
  status = Tcl_GetInt (interp, argv[2], &yr);
  if (status != TCL_OK)
    return status;
  trees_csv_export (global_doclist, argv[3],
                    trees_tcl_glue_exporter, (void *) channel,
                    yr, 3, measurement_defaults);
  return TCL_OK;
}  


int
Sadie_TREES_list_to_vecCmd (ClientData clientData, Tcl_Interp *interp,
                            int objc, Tcl_Obj * const objv[])
{
  widthvecp_t newvec = NULL;
  char *document = NULL;
  char *vecname = NULL;
  double *newvals, *val;
  int len, objcount, i;
  Tcl_Obj **objlist = NULL;
  Tcl_Obj **list_item = NULL;
  int status = TCL_OK;
  int err = 0;

  logerr_assert (LOGERR_CRIT, LOGERR_FUNC, (interp && objv));
  if (objc != 4)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "doc vecname list");
      return TCL_ERROR;
    }
  document = Tcl_GetStringFromObj(objv[1], &len);
  if (len == 0)
    return TCL_ERROR;
  vecname = Tcl_GetStringFromObj(objv[2], &len);
  if (len == 0)
    return TCL_ERROR;
  status = Tcl_ListObjGetElements(interp, objv[3], &objcount, &objlist);
  if (status != TCL_OK)
    return status;
  logerr_assert (LOGERR_DEBUG, LOGERR_FUNC, (objcount >= 0));
  if (objcount == 0)
    newvals = NULL;
  else
    {
      newvals = calloc (objcount, sizeof (double));
      logerr_assert (LOGERR_CRIT, LOGERR_FUNC, (newvals != NULL));
    }
  for (i = objcount, list_item = objlist, val = newvals;
       (status == TCL_OK) && (i != 0);
       i--, list_item++, val++)
    {
      status = Tcl_GetDoubleFromObj (interp, *list_item, val);
    }
  if (status != TCL_OK)
    {
      free (newvals);
      return status;
    }
  newvec = malloc (sizeof (struct Widthvec));
  logerr_assert (LOGERR_CRIT, LOGERR_FUNC, (newvec != NULL));
  newvec->len = objcount;
  newvec->val = newvals;
  err = sad_put_doc_property (global_doclist, document, vecname,
                              widthvec_destruct, newvec);
  if (err)
    {
      free (newvals);
      free (newvec);
    }
  return status;
}

int
Sadie_TREES_open_session_docCmd (ClientData clientData, Tcl_Interp *interp,
                                 int objc, Tcl_Obj * const objv[])
{
  int len;
  char *document = NULL;
  int err = 0;

  logerr_assert (LOGERR_CRIT, LOGERR_FUNC, (interp && objv));
  if (objc != 2)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "doc");
      return TCL_ERROR;
    }
  document = Tcl_GetStringFromObj(objv[1], &len);
  if (len == 0)
    return TCL_ERROR;
  err = sad_put_doc (global_doclist, document, 4, NULL);
  return (err) ? TCL_ERROR : TCL_OK;
}

int
Sadie_TREES_close_session_docCmd (ClientData clientData, Tcl_Interp *interp,
                                 int objc, Tcl_Obj * const objv[])
{
  int len;
  char *document = NULL;
  int err = 0;

  logerr_assert (LOGERR_CRIT, LOGERR_FUNC, (interp && objv));
  if (objc != 2)
    {
      Tcl_WrongNumArgs (interp, 1, objv, "doc");
      return TCL_ERROR;
    }
  document = Tcl_GetStringFromObj(objv[1], &len);
  if (len == 0)
    return TCL_ERROR;
  err = sad_dispose_doc (global_doclist, document);
  return (err) ? TCL_ERROR : TCL_OK;
}

/*************************************************************************
 *
 * This procedure initializes the the Sadie_Proto procedures when
 * this package is loaded.
 *
 *************************************************************************/
int Sadie_TREES_Init(Tcl_Interp *interp) {
  Tcl_CreateCommand(interp, "Sadie_TREES_CreateMosaic", Sadie_TREES_CreateMosaicCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_GetRange", Sadie_TREES_GetRangeCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_ByteSave", Sadie_TREES_ByteSaveCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_ByteOpen", Sadie_TREES_ByteOpenCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_ByteClose", Sadie_TREES_ByteCloseCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_MosaicIndexSave", Sadie_TREES_MosaicIndexSaveCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_MosaicIndexOpen", Sadie_TREES_MosaicIndexOpenCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_MosaicIndexClose", Sadie_TREES_MosaicIndexCloseCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_ChainClose", Sadie_TREES_ChainCloseCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_ByteMosaicResamp", Sadie_TREES_ByteMosaicResampCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_MosaicImport", Sadie_TREES_MosaicImportCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_ByteMosaicExport", Sadie_TREES_ByteMosaicExportCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_Chain2Disk", Sadie_TREES_Chain2DiskCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_Disk2Chain", Sadie_TREES_Disk2ChainCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_ScaleChain", Sadie_TREES_ScaleChainCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_DisplayRings", Sadie_TREES_DisplayRingsCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_ComputeIndices", Sadie_TREES_ComputeIndicesCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_DisplayPartRings", Sadie_TREES_DisplayPartRingsCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_UpdateROI", Sadie_TREES_UpdateROICmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_Chain_RingWidths", Sadie_TREES_Chain_RingWidthsCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_QueryRingType", Sadie_TREES_QueryRingTypeCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_QueryBoundaryId", Sadie_TREES_QueryBoundaryIdCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_Mosaic2MosaicResampl", Sadie_TREES_Mosaic2MosaicResamplCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_IndexResampl", Sadie_TREES_IndexResamplCmd, 
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_ExportRwl", Sadie_TREES_ExportRwlCmd,
        (ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_TREES_ExportCsv", Sadie_TREES_ExportCsvCmd,
        (ClientData) NULL, NULL);
  Tcl_CreateObjCommand (interp, "Sadie_TREES_list_to_vec", Sadie_TREES_list_to_vecCmd,
                        (ClientData) NULL, NULL);
  Tcl_CreateObjCommand (interp, "Sadie_TREES_open_session_doc", Sadie_TREES_open_session_docCmd,
                        (ClientData) NULL, NULL);
  Tcl_CreateObjCommand (interp, "Sadie_TREES_close_session_doc", Sadie_TREES_open_session_docCmd,
                        (ClientData) NULL, NULL);

  Sadie_TREES_Ext_Init(interp);
  
  return TCL_OK;
}

