#ifndef CHAIN_H
#define CHAIN_H 1
#include  "sadie_byte.h"
#include  "mosaic.h"
#include  "trees_profile.h"
#include "project_utilities.h"

BEGIN_C_DECLARATIONS

/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   Macro definitions for chain codes.                                       */
/*                                                                            */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#define  NORMAL   0
#define  ADDED    1
#define  REMOVED  2

#define  TERM_CODE           9
#define  TERM_VALID          0
#define  CHAIN_HEADER_SIZE   512


/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   Data structures to store chain codes.                                    */
/*                                                                            */
/*                                                                            */
/*----------------------------------------------------------------------------*/
typedef struct chain_node {
    unsigned char  code;            /* Code indicating orientation of current
				       pixel wrt previous pixel in the chain. */
    unsigned char  valid;           /* Tag indicating pixel validity.         */
    struct chain_node *next;        /* Pointer to next pixel node.            */
}                       CHAIN_NODE;


typedef struct list_node {
    unsigned int  startj;           /* Line coordinate of first ring pixel.   */
    unsigned int  startk;           /* Pixel coordinate of first ring pixel.  */
    unsigned char ring_type;        /* Number representing boundary type.     */
    int           ring_id;          /* Boundary ID.                           */
    CHAIN_NODE         *chain;      /* Pointer to chain code of the ring.     */
    struct list_node   *next;       /* Pointer to chain code of next ring.    */
}                       LIST_NODE;	


typedef struct {
    unsigned int  nlin;             /* Number of lines in image.              */
    unsigned int  npix;             /* Number of pixels/line.                 */
    char          header[504];      /* Header space for future expansion.     */
    LIST_NODE     *list;            /* Pointer to the list of chains.         */
}                       CHAIN;


/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   Function declarations.                                                   */
/*                                                                            */
/*                                                                            */
/*----------------------------------------------------------------------------*/
void RELMEM_CHAIN (CHAIN *);
void demolish_chainlist (LIST_NODE ** thelisthandle);
void demolish_chainmap (CHAIN ** thechainmaphandle);
LIST_NODE * copy_chainlist (LIST_NODE * oldlist);
void copy_chainlist_head (LIST_NODE * oldhead, LIST_NODE ** newhandle);
void search_chainlist_by_id (int id, LIST_NODE * thelist,
                             LIST_NODE ** pred_handle,
                             LIST_NODE ** match_handle, int * indexpos);
void CHAIN2DISK (CHAIN *,char *);
void SCALE_CHAIN (CHAIN *, int, CHAIN **);
void DISK2CHAIN (char *, CHAIN **);
void CHAIN_COMPUTE_COORDINATES (int, int, unsigned char, int *, int *);
void CHAIN_COMPUTE_CODE (int , int , int , int , unsigned char *);
void CHAIN_RINGWIDTHS (IMAGE_BYTE  *, MOSAIC_INDEX *, CHAIN  *, PROFILE_LIST *, int *, double **, BOUNDBOX_LIST *);

END_C_DECLARATIONS

#endif /* !CHAIN_H */
