#ifndef ADD_RINGWIDTHS_H
#define ADD_RINGWIDTHS_H 1
#include "mosaic.h"
#include "chain.h"
#include "trees_widthvec.h"
#include "trees_profile.h"
#include "boundary.h"
#include "project_utilities.h"

BEGIN_C_DECLARATIONS

/* A value that will never be a valid LIST_NODE ring_id. */
#define INVALID_ID INT_MIN

void provisionalrings_destruct (void *data);

void scale_boundary (BOUNDER * bound, double scale);

LIST_NODE * make_chain (BOUNDER * bound);

void chain_extrapolate (IMAGE_BYTE * mimage, MOSAIC_INDEX * mindex,
                        double scale, double x, double y,
                        double orig_x, double orig_y, double orig_phi,
                        LIST_NODE * guide, BOUNDBOX_LIST box,
                        double *width, PROFILE_LIST * prof,
                        LIST_NODE ** extrapolated,
                        LIST_NODE ** small_extrapolated);

void chain_interpolate (IMAGE_BYTE * mimage, MOSAIC_INDEX * mindex,
                        double scale, double x, double y,
                        LIST_NODE * inner, LIST_NODE * outer,
                        BOUNDBOX_LIST box, double *w_before,
                        PROFILE_LIST * prof_before, double *w_after,
                        PROFILE_LIST * prof_after, LIST_NODE ** interpolated,
                        LIST_NODE ** small_interpolated);

void chain_collapse (IMAGE_BYTE * mimage, MOSAIC_INDEX * mindex,
                     LIST_NODE * inner, LIST_NODE * outer, double *width,
                     PROFILE_LIST * prof);

int provisional_point_in_roi (MOSAIC_INDEX * mindex, double x, double y);

double provisional_point_distance (double x, double y, LIST_NODE * boundary);

int provisional_insert (char *doc, int provbnum, IMAGE_BYTE * mimage,
                        MOSAIC_INDEX * mindex, double scale,
                        double x, double y,
                        LIST_NODE * inner, LIST_NODE * outer);

int provisional_extend (char *doc, int provbnum, IMAGE_BYTE * mimage,
                        MOSAIC_INDEX * mindex, double scale,
                        double x, double y,
                        LIST_NODE * initial, LIST_NODE * final);

int provisional_delete (char *doc, int provbnum, IMAGE_BYTE * mimage,
                        MOSAIC_INDEX * mindex, LIST_NODE * inner,
                        LIST_NODE * deleted, LIST_NODE * outer,
                        LIST_NODE * small);

int provisional_check (double x, double y,
                       LIST_NODE * inner, LIST_NODE * outer);

int have_provisional_changes (char * doc, char * pname);

LIST_NODE * get_provisional_full_chain (char * doc, char * pname);

LIST_NODE * get_provisional_small_chain (char * doc, char * pname);

int get_provisional_insert_point (char * doc, char * pname);

int find_provisional_widths (char * doc, char * pname, double * * widthhandle);

int find_provisional_profiles (char * doc, char * pname,
                               PROFILE_LIST * * profhandle);

int find_provisional_diff_profiles (char * doc, char * pname,
                                    PROFILE_LIST * * diffprofhandle);

int find_provisional_earlywidths (char * doc, char * pname,
                                  double * * earlywidthhandle);

int find_provisional_latewidths (char * doc, char * pname,
                                 double * * latewidthhandle);

int find_provisional_boundbox (char * doc, char * pname,
                               BOUNDBOX_LIST * boundboxhandle);

END_C_DECLARATIONS

#endif /* !ADD_RINGWIDTHS_H */
