#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        <sadie.h>
#include        "sadie_byte.h"
#include        "mosaic.h"
#include        "trees.h"
#include        "math.h"


/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function assembles multiple images into                             */
/*   a single composite mosaic.                                               */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void CREATETREESMOSAIC (
char **imglist, /*  I   Pointer to an array[nimg] containing pointers
		        to the input image names.                             */
int illum_adjust, /*  I   Indicates whether to adjust for illumination
		          distortion.                                         */
char *illum_image,
int nimg,       /*  I   Number of input images.                               */
int nlin,       /*  I   Number of lines in the output mosaic.                 */
int npix,       /*  I   Number of pixels/line in the output mosaic.           */
int frame_nlin, /*  I   Number of lines in each frame.                        */
int frame_npix, /*  I   Number of pixels/line in each frame.                  */
int *xoff,      /*  I   Address of an array[nimg], containing the starting    */
                /*      line coordinates of the input images in the output.   */
int *yoff,      /*  I   Address of an array[nimg], containing the starting    */
                /*      pixel coordinates of the input images in the output.  */
PIXEL *bias,    /*  I   Address of an array[nimg], containing the bias        */
                /*      offsets of the input images relative to the output.   */
PIXEL *gain,    /*  I   Address of an array[nimg], containing the gain        */
                /*      offsets of the input images relative to the output.   */
PIXEL gmin,     /*  I   Minimum value for image pixels after gain/bias
		        adjustments.                                          */
PIXEL factor,   /*  I   Factor to multiply times each pixels.                 */
PIXEL glev,     /*  I   Graylevel to fill otherwise empty pixels.             */
IMAGE_BYTE **out,/*  O   Address of a pointer to the output byte mosaic.      */
MOSAIC_INDEX **index /*  O   Address of a pointer to the output mosaic index. */
/*----------------------------------------------------------------------------*/
) { register int i, j, k, l, elin, epix, lin;
    int maxheight=0,top,bottom, height_actual;
    char msg[SLEN];
    IMAGE* inimg=NULL;
    IMAGE* illum=NULL;
    PIXEL pixval;

    int progmeter;
    double progress;
    double localpercent;
    double localprogress;
    int count, total, increment;
    int cancel=0;

    progmeter = ProgMeter_Create("Create TREES Mosaic");
    progress = 0.0;


    /* if (TIMES) TIMING(T_MOSAIC); */
/*    if (NAMES) { */
	MESSAGE('I',"");
	MESSAGE('I',"----------");
	MESSAGE('I',"");
        MESSAGE('I',"CREATETREESMOSAIC");
        MESSAGE('I',"");
        sprintf(msg," Number of input images:  %d",nimg);
        MESSAGE('I',msg);
        sprintf(msg," Mosaic size: lines:      %d",nlin);
        MESSAGE('I',msg);
        sprintf(msg,"             pixels:      %d",npix);
        MESSAGE('I',msg);
        sprintf(msg," Frame size:  lines:      %d",frame_nlin);
        MESSAGE('I',msg);
        sprintf(msg,"             pixels:      %d",frame_npix);
        MESSAGE('I',msg);
        for (i=0; i<nimg; i++) {
            sprintf(msg," Image %d:   Offset: lines: %-4d pixels: %-4d",i,yoff[i],xoff[i]);
            MESSAGE('I',msg);
            sprintf(msg,"                   Contrast Adjustment: gain: %12.4e bias: %12.4e",gain[i],bias[i]);
            MESSAGE('I',msg);
        }
        sprintf(msg," Minimum input graylevel:       %12.4e",gmin);
        MESSAGE('I',msg);
        sprintf(msg," Factor:       %12.4e",factor);
        MESSAGE('I',msg);
        sprintf(msg," Fill graylevel:       %12.4e",glev);
        MESSAGE('I',msg);
        MESSAGE('I'," ...............");
        MESSAGE('I',"");
/*    } */

    for (i=0; i<nimg; i++) {
        printf(" Image %d:   Offset: lines: %-4d pixels: %-4d \n",i,yoff[i],xoff[i]);
        printf("                   Contrast Adjustment: gain: %12.4e bias: %12.4e \n",gain[i],bias[i]);
    }


    /* create mosaic index of appropriate size */
    if (!CHECKIMG_MOSAIC_INDEX(*index)) GETMEM_MOSAIC_INDEX(nlin,npix,nimg,index);
    if (!*index) goto the_end;
    
    if (progmeter) {
        localpercent = 0.1;
        localprogress = 0.0;
        maxheight = 0;
        increment = rint((double)npix/5.0);
    }
    for (k=0; k<npix; k++) {
        if (progmeter) {
            if ((k%increment) == 0) {
                localprogress = ((double)k/(double)npix)*100.0;
                cancel = ProgMeter_Update(progmeter,(localprogress*localpercent)+progress);
                if (cancel != 0) goto the_end;
            }
        }        

        top = nlin-1;
        bottom = 0;
        for (i=0; i<nimg; i++) {
            if ((xoff[i] <= k) && (xoff[i]+frame_npix > k)) {
                if (yoff[i] < top) top = yoff[i];
                if (yoff[i] + (frame_nlin-1) > bottom) bottom = yoff[i] + (frame_nlin-1);
            }
        }
        (*index)->voffset[k] = top;
        (*index)->vheight[k] = (bottom - top) + 1;
	(*index)->vstart[k] = (*index)->voffset[k];
	(*index)->vend[k] = (*index)->voffset[k] + (*index)->vheight[k] - 1;
	
        if ((*index)->vheight[k] > maxheight) maxheight = (*index)->vheight[k];
    }
    if (progmeter) {
        progress += localpercent * 100.0;
    }    

    (*index)->nframe = nimg;
    for (i=0; i<nimg; i++) {
        (*index)->frame[i].startx = xoff[i];
        (*index)->frame[i].endx = (xoff[i]+frame_npix)-1;
        (*index)->frame[i].starty = yoff[i];
        (*index)->frame[i].endy = (yoff[i]+frame_nlin)-1;
    }


    (*index)->roi->left[0] = 0;
    (*index)->roi->left[1] = 0;
    (*index)->roi->top[0] = 0;
    (*index)->roi->top[1] = (*index)->npix - 1;;
    height_actual = (abs((*index)->frame[nimg-1].starty - (*index)->frame[0].endy) >
		     abs((*index)->frame[nimg-1].endy - (*index)->frame[0].starty)     ?
		     abs((*index)->frame[nimg-1].starty - (*index)->frame[0].endy) :
		     abs((*index)->frame[nimg-1].endy - (*index)->frame[0].starty)    );

    (*index)->roi->right[0] = height_actual;
    (*index)->roi->right[1] = (*index)->npix - 1;;
    (*index)->roi->bottom[0] = height_actual;
    (*index)->roi->bottom[1] = 0;
    
    printf("index->nframe = %d\n",(*index)->nframe);
    for (i=0; i<(*index)->nframe; i++) {
	printf("Frame %d: (%d,%d) to (%d,%d)\n",i,(*index)->frame[i].startx,(*index)->frame[i].starty,(*index)->frame[i].endx,(*index)->frame[i].endy);
    }

    (*index)->height_max = maxheight;
    
    /* create mosaic image of appropriate size */
    if (!CHECKIMG_BYTE(*out)) GETMEM_BYTE(1,maxheight,npix,out);
    if (!*out) goto the_end;

    /* fill the background */
    if (progmeter) {
        localpercent = 0.2;
        localprogress = 0.0;
        count = 0;
        total = (*out)->npix * (*out)->nlin;
        increment = rint((double)total/10.0);
    }
    for (i=0; i<1; i++) {
        for (j=0; j<(*out)->nlin; j++) {
            for (k=0; k<(*out)->npix; k++) {
                if (progmeter) {
                    count++;
                    if ((count%increment) == 0) {
                        localprogress = ((double)count/(double)total)*100.0;
                        cancel = ProgMeter_Update(progmeter,(localprogress*localpercent)+progress);
                        if (cancel != 0) goto the_end;
                    }
                }
                (*out)->data[i][j][k] = glev;
            }
        }
    }
    if (progmeter) {
        progress += localpercent * 100.0;
    }

    if (illum_adjust) {
        DISK2IMG (illum_image,0,0,&illum);
    }

    /* assemble the mosaic */
    for (l=0; l<nimg; l++) {
        DISK2IMG (imglist[l],0,0,&inimg);
        if (!CHECKIMG(inimg)) {        
            sprintf(msg," Cannot open image: %s",imglist[l]);
            MESSAGE('E',msg);
            goto the_end;
        } else if (inimg->nlin != frame_nlin || inimg->npix != frame_npix) {
            sprintf(msg," Input image %s dimensions do not match expected values.",imglist[l]);
            MESSAGE('E',msg);
            MESSAGE('E',"   Generated mosaic may be in error!");
        }

        elin = min(yoff[l]+inimg->nlin,nlin);
        epix = min(xoff[l]+inimg->npix,npix);
        if (progmeter) {
            localpercent = 0.7/(double)nimg;
            count = 0;
            total = (elin-yoff[l])*(epix-xoff[l]);
            increment = rint((double)total/3.0);
        }
        if (illum_adjust) {
            for (j=yoff[l]; j<elin; j++) {
                for (k=xoff[l]; k<epix; k++) {
                    if (progmeter) {
                        count++;
                        if ((count%increment) == 0) {
                            localprogress = ((double)count/(double)total)*100.0;
                            ProgMeter_Update(progmeter,(localprogress*localpercent)+progress);
                        }
                    }

                    lin = j-(*index)->voffset[k];
                    if (lin>=0 && lin<(*out)->nlin) {
                        pixval = (PIXEL)((double)gain[l]*((double)inimg->data[0][j-yoff[l]][k-xoff[l]] / (double)illum->data[0][j-yoff[l]][k-xoff[l]])+(double)bias[l]);
                        (*out)->data[0][lin][k] = (PIXEL_BYTE)rnd((pixval-gmin)*factor) + (PIXEL_BYTE)1;
                    }
                }
            }
        } else {
            for (j=yoff[l]; j<elin; j++) {
                for (k=xoff[l]; k<epix; k++) {
                    if (progmeter) {
                        count++;
                        if ((count%increment) == 0) {
                            localprogress = ((double)count/(double)total)*100.0;
                            cancel = ProgMeter_Update(progmeter,(localprogress*localpercent)+progress);
                            if (cancel != 0) goto the_end;
                        }
                    }

                    lin = j-(*index)->voffset[k];
                    if (lin>=0 && lin<(*out)->nlin) {
                        (*out)->data[0][lin][k] = (PIXEL_BYTE)rnd((((double)gain[l]*(double)inimg->data[0][j-yoff[l]][k-xoff[l]]+(double)bias[l])-gmin)*(double)factor) + (PIXEL_BYTE)1;
                    }
                }
            }
        }
        if (progmeter) {
            progress += localpercent * 100.0;
        }        

        RELMEM(inimg);
        inimg = NULL;
    }

    the_end:
    /* if (TIMES) TIMING(T_EXIT); */
    
    if (progmeter) {
        ProgMeter_Destroy(progmeter);
    }

    if (inimg) RELMEM(inimg);
    if (illum) RELMEM(illum);
    
    if (cancel != 0) {
        if (*out) RELMEM_BYTE(*out);
        *out = NULL;
        if (*index) RELMEM_MOSAIC_INDEX(*index);
        *index = NULL;
    }
    
    return;
}
