/* Enable the SADIE libraries to be linked to C++ code */
#ifdef __cplusplus
extern "C" {
#endif 

#ifndef MOSAIC_H
#define MOSAIC_H

#include "sadie_byte.h"

    
/*----------------------------------------------------------------------------*/
/*    NOTE: When converting between a normal image and a mosaic image:        */
/*          j_mosaic = j_normal - voffset[k]                                  */
/*----------------------------------------------------------------------------*/

/* Data structure to store the user-defined region-of-interest co-ordinates.  */
/* Currently restricted to a rectangular region with arbitrary orientation.   */
/* This forms a part of the MOSAIC_INDEX data structure.                      */
typedef struct {
    uint32_t  left[2];            /* Co-ordinates of left extreme point
					   in ROI                             */
    uint32_t  right[2];           /* Co-ordinates of right extreme point
					   in ROI                             */
    uint32_t  top[2];             /* Co-ordinates of top extreme point
					   in ROI                             */
    uint32_t  bottom[2];          /* Co-ordinates of bottom extreme
					   point in ROI                       */
}                       REGION_OF_INTEREST;
    

/* Data structure to store the bounding rectangle for the frames.             */
/* This forms a part of the MOSAIC_INSEX data structure                       */
typedef struct {
    uint32_t  startx;
    uint32_t  endx;
    uint32_t  starty;
    uint32_t  endy;
}                       MOSAIC_FRAME;


/* Data structure to store the index values for a mosaic image.               */
/* This information is vital to retrieve the raster image from the mosaic.    */
/* Refer to Conner's thesis for more information.                             */
typedef struct {
    uint32_t  nlin;               /* Number of lines in mosaic          */
    uint32_t  npix;               /* Number of pixels/line              */
    uint32_t  height_max;         /* Maximum number of pixels in a
					   given line                         */
    uint32_t  nframe;                      /* Number of frames in the mosaic     */
    uint32_t  *voffset;           /* Pointer to offset of each column
					   of data                            */
    uint32_t  *vheight;           /* Pointer to the height of each
					   column of data                     */
    uint32_t  hstart;             /* X co-ordinate of left extreme
					   point in ROI                       */
    uint32_t  hend;               /* X co-ordinate of right extreme
					   point in ROI                       */
    uint32_t  *vstart;            /* Pointer to the start of ROI for
					   each column of data                */
    uint32_t  *vend;              /* Pointer to the end of ROI for each
					   column of data                     */
    REGION_OF_INTEREST *roi;      /* Pointer to the co-ordinates of ROI */
    MOSAIC_FRAME *frame;                /* Position and size of each frame    */
}                       MOSAIC_INDEX;



/*----------------------------------------------------------------------------*/

extern int   CHECKIMG_MOSAIC_INDEX (MOSAIC_INDEX *);
extern void  RELIMG_MOSAIC_INDEX (MOSAIC_INDEX **);
extern void  RELMEM_MOSAIC_INDEX (MOSAIC_INDEX *);
extern void  GETMEM_MOSAIC_INDEX (int, int, uint32_t, MOSAIC_INDEX **);

extern void  MOSAIC_DISK2INDEX (unsigned char *, MOSAIC_INDEX **);
extern void  MOSAIC_INDEX2DISK (MOSAIC_INDEX *, unsigned char *);

extern void  MOSAIC_BYTE_RESAMPL (IMAGE_BYTE *, MOSAIC_INDEX *, uint32_t, uint32_t, uint32_t, uint32_t, IMAGE **);
extern void  MOSAIC_BYTE_EXTRACT (IMAGE_BYTE *,MOSAIC_INDEX *, int, int, int, int, PIXEL,IMAGE **);
extern void  MOSAIC_BYTE_INSERT (IMAGE *, MOSAIC_INDEX *, int, int, IMAGE_BYTE **);
extern void  MOSAIC_BYTE_EXPORT (IMAGE_BYTE*, char*);
extern void  MOSAIC_IMPORT (MOSAIC_INDEX *, char *, int,int ,int , int,int ,int , IMAGE **);

extern void  MOSAIC2MOSAIC_RESAMPL (IMAGE_BYTE *, MOSAIC_INDEX *, uint32_t, uint32_t, uint32_t, uint32_t, IMAGE_BYTE **);
extern void  INDEX_RESAMPL (MOSAIC_INDEX *, int, MOSAIC_INDEX **);
extern void  MOSAIC_BYTE_TRACKRINGS (IMAGE_BYTE *, MOSAIC_INDEX *, IMAGE_BYTE **, IMAGE_BYTE **);

#endif

#ifdef __cplusplus
}
#endif
