#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        <sadie.h>
#include        "sadie_byte.h"
#include        "mosaic.h"

/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function extracts a window from a byte mosaic image.                */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void MOSAIC_BYTE_EXTRACT (
IMAGE_BYTE *in, /*  I   Pointer to the input mosaic image.                    */
MOSAIC_INDEX *index, /*  I   Pointer to the input mosaic index.                    */
int jbgn,     /*  I   Starting line.                                        */
int jend,     /*  I   Ending line.                                          */
int kbgn,     /*  I   Starting pixel.                                       */
int kend,     /*  I   Ending pixel.                                         */
PIXEL fill,     /*  I   Value to fill non-existent pixels with.               */
IMAGE **out     /*  O   Address of a pointer to the output image.             */
/*----------------------------------------------------------------------------*/
) { size_t j_0, k_0;
    int j, k, row_lo, row_hi;
    char   msg[SLEN];

    printf("extracting (%d,%d) to (%d,%d)\n",kbgn,jbgn,kend,jend);

    /* check input */
    if (!CHECKIMG_BYTE(in)) {
        printf("Can't identify mosaic index.\n");
        MESSAGE('E'," Can't identify mosaic image.");
        goto the_end;
    } else if (!CHECKIMG_MOSAIC_INDEX(index)) {
        printf("Can't identify mosaic index\n");
        MESSAGE('E'," Can't identify mosaic index.");
        goto the_end;
    } else if (index->npix != in->npix) {
        printf("index->npix = %d, in->npix = %d\n",index->npix, in->npix);
        printf("Size of mosaic image and mosaic index do not match.\n");
        MESSAGE('E'," Size of mosaic image and mosaic index do not match.");
        goto the_end;
    } else if (in->nbnd > 1) {
        printf("Image must be single-band.\n");
        MESSAGE('E'," Image must be single-band.");
        goto the_end;
    } else if (jbgn > jend) {
        printf("Start line must be before end line.\n");
        MESSAGE('E'," Start line must be before end line.");
        goto the_end;
    } else if (kbgn > kend) {
        printf("Start pixel must be before end pixel.\n");
        MESSAGE('E'," Start pixel must be before end pixel.");
        goto the_end;
    }

    /* if (TIMES) TIMING(T_SCONVL); */
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"MOSAIC_BYTE_EXTRACT");
        MESSAGE('I',"");
        sprintf(msg," Input image:           %s",in->text);
        MESSAGE('I',msg);
        sprintf(msg,"  Start line:    %d",jbgn);
        MESSAGE('I',msg);
        sprintf(msg,"    End line:    %d",jend);
        MESSAGE('I',msg);
        sprintf(msg," Start pixel:    %d",kbgn);
        MESSAGE('I',msg);
        sprintf(msg,"   End pixel:    %d",kend);
        MESSAGE('I',msg);
        MESSAGE('I'," ...............");
    }


    /* create image of appropriate size */
    if (!CHECKIMG(*out)) GETMEM(1,(jend-jbgn)+1,(kend-kbgn)+1,out);
    if (!*out) goto the_end;

    for (j_0=0; j_0<(*out)->nlin; j_0++) {
        for (k_0=0; k_0<(*out)->npix; k_0++) {
            (*out)->data[0][j_0][k_0] = (PIXEL)fill;
        }
    }
    
    for (k=kbgn; k<=kend; k++) {
        if (k>0  &&  k<in->npix) {
            row_lo = jbgn - index->voffset[k];
            row_hi = jend - index->voffset[k];
            for (j=row_lo; j<=row_hi; j++) {
                if (j>0 && j<index->vheight[k]) {
                    (*out)->data[0][j-row_lo][k-kbgn] = (PIXEL)in->data[0][j][k];
                }
            }
        }
    }

    the_end:
    return;
    /* if (TIMES) TIMING(T_EXIT); */
}
