/**
 *  TREES brightness profiles (generally for single rings)
 */

#ifndef TREES_PROFILE_H
#define TREES_PROFILE_H 1

#include "project_utilities.h"
#include "trees_widthvec.h"

BEGIN_C_DECLARATIONS

/**
 *  Brightness value representation
 */
typedef double BRIGHTNESS;

#define DARKNESS ((BRIGHTNESS)0)

/**
 *  Individual brightness profiles.
 *  Even an individual profile is structured as a list node.
 */
typedef struct profile_node {
  struct profile_node *next; /**< Link to next node in list. */
  int len;         /**< Profile length for this node. */
  BRIGHTNESS *val; /**< Points to a extensible array of brigthnesses. */
} PROFILE_NODE;

typedef PROFILE_NODE* PROFILE_LIST;

typedef struct boundbox_node {
  struct boundbox_node *next; /**< Link to next node in list. */
  double minx;         /**< minimum x coordinate.*/
  double miny;        /**< minimum y coordinate.*/
  double maxx;       /**< maximum x coordinate.*/
  double maxy;      /**< maximum y coordinate.*/
} BOUNDBOX_NODE;

typedef BOUNDBOX_NODE* BOUNDBOX_LIST;

PROFILE_LIST new_profile (int);
void demolish_profiles (PROFILE_LIST);
void demolish_boundbox (BOUNDBOX_LIST);
PROFILE_LIST reduce_profiles (PROFILE_LIST);
int find_mean_profile(PROFILE_LIST, PROFILE_LIST);
void First_diff_profiles(PROFILE_LIST *profiles, PROFILE_LIST *diffprofiles,
                         widthvecp_t ringwidths,
                         widthvecp_t earlyringwidth,
                         widthvecp_t lateringwidth);

END_C_DECLARATIONS
#endif
