#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        <sadie.h>
#include        "trees.h"

/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*  Description:  This procedure trims an image.                                                       */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void TRIM (
IMAGE  *in,     /*  I   Pointer to the input image.                           */
uint32_t    trimwidth,
IMAGE  **out    /*  O   Address of a pointer to the output image.             */
/*----------------------------------------------------------------------------*/
) { register uint32_t j, k;
    char   msg[SLEN];

    /* check input */
    if (!CHECKIMG(in)) {
        MESSAGE('E'," Can't identify image.");
        goto the_end;
    } else if (in->nbnd > 1) {
        MESSAGE('E'," Input image must be single band.");
        goto the_end;
    } else if (2*trimwidth >= in->npix || 2*trimwidth >= in->nlin) {
        MESSAGE('E'," Trim width is too large for image size.");
        goto the_end;
    }


    /* if (TIMES) TIMING(T_SIZEFILTER); */
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"TRIM");
        MESSAGE('I',"");
        sprintf(msg," Input image:                      %s",in->text);
        MESSAGE('I',msg);
        sprintf(msg," Trim width:                   %d",trimwidth);
        MESSAGE('I',msg);
        MESSAGE('I'," ...............");
    }

    

    /* create output image of appropriate size */
    if (!CHECKIMG(*out)) GETMEM(1,in->nlin-(2*trimwidth),in->npix-(2*trimwidth),out);
    if (!*out) goto the_end;


    /* Create the output image */
    for (j=trimwidth; j<in->nlin-trimwidth; j++) {
        for (k=trimwidth; k<in->npix-trimwidth; k++) {
            (*out)->data[0][j-trimwidth][k-trimwidth] = in->data[0][j][k];
        }
    }


    the_end:

    /* if (TIMES)   TIMING(T_EXIT); */
    return;
}
