#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include <sadie.h>
#include "proto.h"
#if WITH_DMALLOC
#include <dmalloc.h>
#endif /* WITH_DMALLOC */

/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/* This function computes a (size x size) Gaussian filter with standard   */
/* deviation sigma.                                                           */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void CREATEGAUSS (
int size,       /*  I   The size of the image.                                */
double sigma,   /*  I   The standard deviation of the gaussian.               */
IMAGE **out     /*  O   Address of a pointer to the output image              */
/*----------------------------------------------------------------------------*/
) { register int i, j, k;
    char msg[SLEN];
    double x, y, *elin=0, *epix=0;

    /* if (TIMES) TIMING(T_GAUSS); */
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"GAUSS");
        MESSAGE('I',"");
        sprintf(msg," Size:                       %i x %i",size,size);
        MESSAGE('I',msg);
        sprintf(msg," Std. Deviation:             %f",sigma);
        MESSAGE('I',msg);
        MESSAGE('I'," ...............");
    }

    /* check parameters */
    if (size < 2  ||  size%2 == 0) {
        MESSAGE('E'," Size of gaussian must be an odd number greater than two.");
        goto the_end;
    } else if (sigma <= 0.0) {
        MESSAGE('E'," Standard deviation of gaussian must be positive.");
        goto the_end;
    }
    
    /* create image of appropriate size */
    if (!CHECKIMG(*out)) GETMEM(1,size,size,out);
    if (!*out) goto the_end;

    /* allocate memory for temporary storage */
    if (!(elin = (double *)malloc(((size/2)+1)*sizeof(double)))) {
        MESSAGE('E'," Memory request failed.");
        goto the_end;
    }
    if (!(epix = (double *)malloc(((size/2)+1)*sizeof(double)))) {
        MESSAGE('E'," Memory request failed.");
        goto the_end;
    }


    /* store one dimensional components */
    for (j=0; j<size/2; j++) {
        y = ((double)(j-size/2)*(double)(j-size/2))/(2*sigma*sigma);
        elin[j] = exp(-y);
    }
    for (k=0; k<size/2; k++) {
        x = ((double)(k-size/2)*(double)(k-size/2))/(2*sigma*sigma);
        epix[k] = exp(-x);
    }
    /* symmetric quadrants */
    for (j=0; j<size/2; j++) {
        for (k=0; k<size/2; k++) {
            (*out)->data[0][j][k] =
            (*out)->data[0][j][size-k-1] =
            (*out)->data[0][size-j-1][k] =
            (*out)->data[0][size-j-1][size-k-1] =
            (PIXEL)(epix[k]*elin[j]);
        }
    }
    /* zero coordinates */
    for (j=0; j<size; j++) {
        y = ((double)(j-size/2)*(double)(j-size/2))/(2*sigma*sigma);
        (*out)->data[0][j][size/2] = (PIXEL)(exp(-y));
        (*out)->data[0][size/2][j] = (PIXEL)(exp(-y));
    }

    	
    the_end:
    if (elin)  free(elin);
    if (epix)  free(epix);
    /* if (TIMES) TIMING(T_EXIT); */
}
