/*
##########################################
# Sadie_FileIO.c -
#   Set of routines for linking SADIE file I/O routines to tcl/tk.
#
# RCS: $Id: Sadie_FileIO.c,v 1.4 2004/09/07 23:35:08 mmunro Exp $ 
# 
# Digital Image Analysis Lab
# Dept. of Electrical and Computer Engineering
# University of Arizona
##########################################
*/

#if HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include <tcl.h>
#include <tk.h>
#include <stdio.h>
#include <string.h>
#include <sadie.h>
#include <math.h>
#if WITH_DMALLOC
#include <dmalloc.h>
#endif /* WITH_DMALLOC */


/* RCS Indentification information */
static char rcsid[] = "$Id: Sadie_FileIO.c,v 1.4 2004/09/07 23:35:08 mmunro Exp $";


int Sadie_FileIO_OpenCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
  int addrstr;
  long filesize;
  IMAGE *theimg=NULL;
  char   msg[SLEN];
  Tcl_Obj* tclvar=NULL;
  if( argc <= 3 ) {
    sprintf(msg, "Usage: Sadie_FileIO_Open filename filesize title");
    Tcl_SetResult(interp, msg, TCL_VOLATILE);
    return TCL_ERROR;
  }

  /* Read the size of the file */
  tclvar = Tcl_NewStringObj(argv[2],-1);
  if (Tcl_GetLongFromObj(interp, tclvar, &filesize) == TCL_ERROR) {
      return TCL_ERROR;
  }
  Tcl_DecrRefCount(tclvar);
  
  DISK2IMG (argv[1],0,filesize,&theimg);
  
  if (theimg) sprintf( theimg->text, "%s", argv[3]);
  addrstr = (int) theimg;

  sprintf(msg, "%x", addrstr);
  Tcl_SetResult(interp, msg, TCL_VOLATILE);

  return TCL_OK;
}







int Sadie_FileIO_SaveCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
  int imgaddr;
  IMAGE *theimg=NULL;
  char* name;
  int bitsperpix;
  char   msg[SLEN];
  

  if( argc <= 3 ) {
    sprintf(msg, "Usage: Sadie_FileIO_Save image_address filename bitsperpixel");
    Tcl_SetResult(interp, msg, TCL_VOLATILE);
    return TCL_ERROR;
  }

  sscanf(argv[1], "%x", &imgaddr);
  theimg = (IMAGE *) imgaddr;

  sscanf(argv[3], "%d", &bitsperpix);

  name = argv[2];


  IMG2DISK (theimg,name,bitsperpix,0,0);
  
  return TCL_OK;
}


int Sadie_FileIO_SaveEPSCmd(ClientData client_data, Tcl_Interp* interp, int argc, char *argv[])
{
  int imgaddr;
  IMAGE *theimg=NULL;
  char* name;
  int bitsperpix;
  char   msg[SLEN];
  

  if( argc <= 3 ) {
    sprintf(msg, "Usage: Sadie_FileIO_SaveEPS image_address filename bitsperpixel");
    Tcl_SetResult(interp, msg, TCL_VOLATILE);
    return TCL_ERROR;
  }

  sscanf(argv[1], "%x", &imgaddr);
  theimg = (IMAGE *) imgaddr;

  sscanf(argv[3], "%d", &bitsperpix);

  name = argv[2];


  IMG2EPSF (theimg,name,bitsperpix,0,0);
  
  return TCL_OK;
}





/*************************************************************************
 *
 * This procedure initializes the the Sadie_FileIO procedures when
 * this package is loaded.
 *
 *************************************************************************/
int Sadie_FileIO_Init(Tcl_Interp *interp) {
  Tcl_CreateCommand(interp, "Sadie_FileIO_Open", Sadie_FileIO_OpenCmd, 
  	(ClientData) NULL, NULL);
  Tcl_CreateCommand(interp, "Sadie_FileIO_Save", Sadie_FileIO_SaveCmd, 
  	(ClientData) NULL, NULL);
 
  return TCL_OK;
}


